subroutine rttov_read_coefs( err, coefs, opts,  &
                             channels,          &
                             channels_rec,      &
                             form_coef,         &
                             form_scaer,        &
                             form_sccld,        &
                             form_pccoef,       &
                             file_coef,         &
                             file_scaer,        &
                             file_sccld,        &
                             file_pccoef,       &
                             file_id_coef,      &
                             file_id_scaer,     &
                             file_id_sccld,     &
                             file_id_pccoef,    &
                             instrument)
! Description:
!   Read coefficients structure
!
! Copyright:
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! Method:
!
! Current Code Owner: SAF NWP
!
! History:
! Version   Date     Comment
! -------   ----     -------
! 2010/03 Code cleaning Pascal Brunel, Philippe Marguinaud
! 2011/11 Introduced PC multiple band and cloudy computations, Marco Matricardi, ECMWF
!
!
! Code Description:
!   Language:           Fortran 90.
!   Software Standards: "European Standards for Writing and
!     Documenting Exchangeable Fortran 90 Code".
#include "throw.h"

Use rttov_types, Only : rttov_coefs, rttov_options
Use parkind1, Only : jpim

!INTF_OFF
Use parkind1, Only : jprb, jplm
use rttov_coef_io_mod, Only : getlun, closelun
use rttov_const, Only : inst_name
use yomhook, Only : LHOOK, DR_HOOK
#ifdef _RTTOV_HDF
use rttov_hdf_mod, Only : &
    open_hdf,             &
    close_hdf,            &
    is_hdf_open,          &
    is_hdf_64bit_reals
#endif
!INTF_ON


Implicit None

Integer(Kind=jpim), Intent(out) :: err
Type(rttov_coefs),  Intent(out) :: coefs
Type(rttov_options),Intent(in)  :: opts
Integer(Kind=jpim), Intent(in), Optional :: channels(:)
Integer(Kind=jpim), Intent(in), Optional :: channels_rec(:)
Character(len=*),   Intent(in), Optional :: form_coef
Character(len=*),   Intent(in), Optional :: form_scaer
Character(len=*),   Intent(in), Optional :: form_sccld
Character(len=*),   Intent(in), Optional :: form_pccoef
Character(len=*),   Intent(in), Optional :: file_coef
Character(len=*),   Intent(in), Optional :: file_scaer
Character(len=*),   Intent(in), Optional :: file_sccld
Character(len=*),   Intent(in), Optional :: file_pccoef
Integer(Kind=jpim), Intent(in), Optional :: file_id_coef
Integer(Kind=jpim), Intent(in), Optional :: file_id_scaer
Integer(Kind=jpim), Intent(in), Optional :: file_id_sccld
Integer(Kind=jpim), Intent(in), Optional :: file_id_pccoef
Integer(Kind=jpim), Intent(in), Optional :: instrument(3)

!INTF_END

#include "rttov_errorreport.interface"
#include "rttov_cmpuc.interface"


#include "rttov_read_ascii_coef.interface"
#include "rttov_read_ascii_pccoef.interface"
#include "rttov_read_ascii_scaercoef.interface"
#include "rttov_read_ascii_sccldcoef.interface"

#include "rttov_read_binary_coef.interface"
#include "rttov_read_binary_pccoef.interface"
#include "rttov_read_binary_scaercoef.interface"
#include "rttov_read_binary_sccldcoef.interface"

#include "rttov_channel_extract_coef.interface"
#include "rttov_channel_extract_scaercoef.interface"
#include "rttov_channel_extract_sccldcoef.interface"
#include "rttov_channel_extract_pccoef.interface"

#include "rttov_check_channels_pc.interface"

#include "rttov_init_coefs.interface"
#include "rttov_nullify_coefs.interface"

#include "rttov_dealloc_coef.interface"
#include "rttov_dealloc_coef_scatt_ir.interface"
#include "rttov_dealloc_optpar_ir.interface"
#include "rttov_dealloc_coef_pccomp.interface"

#ifdef _RTTOV_HDF
#include "rttov_hdf_load.interface"
#endif

Integer(Kind=jpim) :: file_id_coef1
Integer(Kind=jpim) :: file_id_scaer1
Integer(Kind=jpim) :: file_id_sccld1
Integer(Kind=jpim) :: file_id_pccoef1

Character(len=256) :: file_coef1
Character(len=256) :: file_scaer1
Character(len=256) :: file_sccld1
Character(len=256) :: file_pccoef1

character(len=32) :: form_coef1
character(len=32) :: form_scaer1
character(len=32) :: form_sccld1
character(len=32) :: form_pccoef1

type(rttov_coefs) :: coefs_all_chan

#ifdef _RTTOV_HDF
Logical(Kind=jplm) :: hdf_was_open, hdf_was_64bit_reals
#endif

character(len=200) :: zeemanmsg
real(kind=jprb)    :: ZHOOK_HANDLE
TRY

!

IF (LHOOK) CALL DR_HOOK('RTTOV_READ_COEFS', 0_jpim, ZHOOK_HANDLE)

CALL rttov_nullify_coefs(coefs)

IF (PRESENT(channels)) CALL rttov_nullify_coefs(coefs_all_chan)


#ifdef _RTTOV_HDF
hdf_was_open = is_hdf_open
hdf_was_64bit_reals = is_hdf_64bit_reals
call open_hdf(.true._jplm, err)
THROW(err.ne.0)
#endif

call getlun(err, file_id_coef1, file_coef1, form_coef1, .false._jplm, "rtcoef", &
               & file_id_coef, file_coef, form_coef, instrument )
THROW(err.ne.0)

if (rttov_cmpuc(form_coef1,"unformatted")) then
  call rttov_read_binary_coef (err, coefs%coef, file_id_coef1, channels)
  THROW(err.ne.0)
  call closelun(err, file_id_coef1, file_id_coef)
  THROW(err.ne.0)
else if (rttov_cmpuc(form_coef1,"formatted")) then
  call rttov_read_ascii_coef (err, coefs%coef, file_id_coef1, channels)
  THROW(err.ne.0)
  call closelun(err, file_id_coef1, file_id_coef)
  THROW(err.ne.0)
else if (rttov_cmpuc(form_coef1,"hdf5")) then
#ifndef _RTTOV_HDF
  err =errorstatus_fatal
  THROWM(err.ne.0,"This program is not compiled with HDF5 capability; use RTTOV_HDF=1 with Makefile.PL")
#else

  IF (PRESENT(channels)) THEN

    CALL rttov_hdf_load(err, file_coef1, "/COEF", coef=coefs_all_chan%coef)
    THROW(err.ne.0)

    CALL rttov_channel_extract_coef(err, coefs_all_chan%coef, coefs%coef, channels)
    THROW(err.ne.0)

  ELSE

    CALL rttov_hdf_load(err, file_coef1, "/COEF", coef=coefs%coef)
    THROW(err.ne.0)

  ENDIF

#endif
else
  err = errorstatus_fatal
  THROWM(err.ne.0,"Unknown format "//Trim(form_coef1))
endif


if( coefs%coef%fmv_chn .le. 0_jpim ) err = errorstatus_fatal
THROWM(err.ne.0,"File does not contain RTTOV coefficients")

if (coefs%coef%inczeeman .and. opts%config%verbose) then
  write (zeemanmsg,"(a,i3,a)") "Zeeman coefficient file for "// &
    trim(inst_name(coefs%coef%id_inst))//" with ",coefs%coef%fmv_var(1)," predictors"
  INFO(zeemanmsg)
end if


!

if (opts%rt_ir%addaerosl .and. .not. opts%rt_ir%user_aer_opt_param) then

  call getlun(err, file_id_scaer1, file_scaer1, form_scaer1, .false._jplm, "scaercoef", &
                &  file_id_scaer,  file_scaer, form_scaer, instrument )
  THROW(err.ne.0)
  if (rttov_cmpuc(form_scaer1,"unformatted")) then
    call rttov_read_binary_scaercoef (err, coefs%coef, coefs%coef_scatt_ir, &
                                      coefs%optp, file_id_scaer1, channels)
    THROW(err.ne.0)
    call closelun(err, file_id_scaer1, file_id_scaer)
    THROW(err.ne.0)
  else if (rttov_cmpuc(form_scaer1,"formatted")) then
    call rttov_read_ascii_scaercoef (err, coefs%coef, coefs%coef_scatt_ir, &
                                     coefs%optp, file_id_scaer1, channels)
    THROW(err.ne.0)
    call closelun(err, file_id_scaer1, file_id_scaer)
    THROW(err.ne.0)
  else if (rttov_cmpuc(form_scaer1,"hdf5")) then
#ifndef _RTTOV_HDF
  err =errorstatus_fatal
  THROWM(err.ne.0,"This program is not compiled with HDF5 capability; use RTTOV_HDF=1 with Makefile.PL")
#else

    IF (PRESENT(channels)) THEN

      CALL rttov_hdf_load(err, file_scaer1, "/SCAER", &
                & scaercoef=coefs_all_chan%coef_scatt_ir, &
                & optp=coefs_all_chan%optp, coef=coefs_all_chan%coef)
      THROW(err.ne.0)

      CALL rttov_channel_extract_scaercoef(err, coefs_all_chan%coef_scatt_ir, coefs_all_chan%optp, &
                                           coefs%coef_scatt_ir, coefs%optp, channels)
      THROW(err.ne.0)

      CALL rttov_dealloc_coef_scatt_ir(err, coefs_all_chan%coef_scatt_ir)
      THROW(err.ne.0)

      CALL rttov_dealloc_optpar_ir(err, coefs_all_chan%optp)
      THROW(err.ne.0)

    ELSE

      CALL rttov_hdf_load(err, file_scaer1, "/SCAER", &
                & scaercoef=coefs%coef_scatt_ir, &
                & optp=coefs%optp, coef=coefs%coef)
      THROW(err.ne.0)

    ENDIF

#endif
  else
    err = errorstatus_fatal
    THROWM(err.ne.0,"Unknown format "//Trim(form_scaer1))
  endif

endif

!

if (opts%rt_ir%addclouds .and. .not. opts%rt_ir%user_cld_opt_param) then

  call getlun(err, file_id_sccld1, file_sccld1, form_sccld1, .false._jplm, "sccldcoef", &
                & file_id_sccld, file_sccld, form_sccld, instrument )
  THROW(err.ne.0)
  if (rttov_cmpuc(form_sccld1,"unformatted")) then
    call rttov_read_binary_sccldcoef (err, coefs%coef, coefs%coef_scatt_ir, &
                                      coefs%optp, file_id_sccld1, channels)
    THROW(err.ne.0)
    call closelun(err, file_id_sccld1, file_id_sccld)
    THROW(err.ne.0)
  else if (rttov_cmpuc(form_sccld1,"formatted")) then
    call rttov_read_ascii_sccldcoef (err, coefs%coef, coefs%coef_scatt_ir, &
                                     coefs%optp, file_id_sccld1, channels)
    THROW(err.ne.0)
    call closelun(err, file_id_sccld1, file_id_sccld)
    THROW(err.ne.0)
  else if (rttov_cmpuc(form_sccld1,"hdf5")) then
#ifndef _RTTOV_HDF
  err =errorstatus_fatal
  THROWM(err.ne.0,"This program is not compiled with HDF5 capability; use RTTOV_HDF=1 with Makefile.PL")
#else

    IF (PRESENT(channels)) THEN

      CALL rttov_hdf_load(err, file_sccld1, "/SCCLD", &
                & sccldcoef=coefs_all_chan%coef_scatt_ir, &
                & optp=coefs_all_chan%optp, coef=coefs_all_chan%coef)
      THROW(err.ne.0)

      CALL rttov_channel_extract_sccldcoef(err, coefs_all_chan%coef_scatt_ir, coefs_all_chan%optp, &
                                           coefs%coef_scatt_ir, coefs%optp, channels)
      THROW(err.ne.0)

      CALL rttov_dealloc_coef_scatt_ir(err, coefs_all_chan%coef_scatt_ir)
      THROW(err.ne.0)

      CALL rttov_dealloc_optpar_ir(err, coefs_all_chan%optp)
      THROW(err.ne.0)

    ELSE

      CALL rttov_hdf_load(err, file_sccld1, "/SCCLD", &
                & sccldcoef=coefs%coef_scatt_ir, &
                & optp=coefs%optp, coef=coefs%coef)
      THROW(err.ne.0)

    ENDIF

#endif
  else
    err = errorstatus_fatal
    THROWM(err.ne.0,"Unknown format "//Trim(form_sccld1))
  endif

endif

!

if (opts%rt_ir%pc%addpc) then

  if (opts%rt_ir%addclouds) then
    call getlun(err, file_id_pccoef1, file_pccoef1, form_pccoef1, .false._jplm, "pccoefcld", &
                   & file_id_pccoef, file_pccoef, form_pccoef, instrument )
  else
    call getlun(err, file_id_pccoef1, file_pccoef1, form_pccoef1, .false._jplm, "pccoef", &
                   & file_id_pccoef, file_pccoef, form_pccoef, instrument )
  endif
  THROW(err.ne.0)
  if (rttov_cmpuc(form_pccoef1,"unformatted")) then
    if (opts%rt_ir%pc%addradrec) then
      call rttov_read_binary_pccoef (err, opts, coefs%coef, coefs%coef_pccomp, &
                                     file_id_pccoef1, channels, channels_rec)
      THROW(err.ne.0)
    else
      call rttov_read_binary_pccoef (err, opts, coefs%coef, coefs%coef_pccomp, &
                                     file_id_pccoef1, channels)
      THROW(err.ne.0)
    endif
    call closelun(err, file_id_pccoef1, file_id_pccoef)
    THROW(err.ne.0)
  else if (rttov_cmpuc(form_pccoef1,"formatted")) then
    if (opts%rt_ir%pc%addradrec) then
      call rttov_read_ascii_pccoef (err, opts, coefs%coef, coefs%coef_pccomp, &
                                    file_id_pccoef1, channels, channels_rec)
      THROW(err.ne.0)
    else
      call rttov_read_ascii_pccoef (err, opts, coefs%coef, coefs%coef_pccomp, &
                                    file_id_pccoef1, channels)
      THROW(err.ne.0)
    endif
    call closelun(err, file_id_pccoef1, file_id_pccoef)
    THROW(err.ne.0)
  else if (rttov_cmpuc(form_pccoef1,"hdf5")) then
#ifndef _RTTOV_HDF
    err =errorstatus_fatal
    THROWM(err.ne.0,"This program is not compiled with HDF5 capability; use RTTOV_HDF=1 with Makefile.PL")
#else

    IF (PRESENT(channels)) THEN

      CALL rttov_hdf_load(err, file_pccoef1, "/PC", pccoef=coefs_all_chan%coef_pccomp, coef=coefs_all_chan%coef)
      THROW(err.ne.0)

      IF (opts%rt_ir%pc%addradrec) THEN
        CALL rttov_channel_extract_pccoef(err, coefs_all_chan%coef_pccomp, &
                                          coefs%coef_pccomp, channels, channels_rec)
      ELSE
        CALL rttov_channel_extract_pccoef(err, coefs_all_chan%coef_pccomp, &
                                          coefs%coef_pccomp, channels)
      ENDIF
      THROW(err.ne.0)

      CALL rttov_dealloc_coef_pccomp(err, coefs_all_chan%coef_pccomp)
      THROW(err.ne.0)

    ELSE

      CALL rttov_hdf_load(err, file_pccoef1, "/PC", pccoef=coefs%coef_pccomp, coef=coefs%coef)
      THROW(err.ne.0)

    ENDIF

#endif
  else
    err = errorstatus_fatal
    THROWM(err.ne.0,"Unknown format "//Trim(form_pccoef1))
  endif

  IF (PRESENT(channels)) THEN
    CALL rttov_check_channels_pc(err, opts, coefs, channels)
    THROW(err.ne.0)
  ENDIF

endif

#ifdef _RTTOV_HDF
! If HDF5 lib was open before rttov_read_coefs was called, make sure the real
! kind is the same as it was previously. Otherwise close the library.
if (hdf_was_open) then
  call open_hdf(hdf_was_64bit_reals, err)
  THROW(err.ne.0)
else
  call close_hdf(err)
  THROW(err.ne.0)
endif
#endif

IF (PRESENT(channels)) THEN
  ! We could call dealloc_coefs here for all sub-structures in coefs_all_chan
  ! but it's more efficient in memory to deallocate them as soon as possible above.
  ! coefs_all_chan%coef is required for reading other coef types so must be
  ! deallocated last.
  CALL rttov_dealloc_coef(err, coefs_all_chan%coef)
  THROW(err.ne.0)
ENDIF


call rttov_init_coefs(err, opts, coefs)
THROWM(err.ne.0,"Failed to initialise coefficients")


IF (LHOOK) CALL DR_HOOK('RTTOV_READ_COEFS', 1_jpim, ZHOOK_HANDLE)
CATCH
IF (LHOOK) CALL DR_HOOK('RTTOV_READ_COEFS', 1_jpim, ZHOOK_HANDLE)
end subroutine
