#!/usr/bin/env python

# ============================================================================
# README
# ============================================================================
#
# This file contains the data on which the RTTOV test profile datasets are based.
# These profiles should be nominally physical, but they are intended primarily
# for technical rather than scientific testing of the code.
#
# The atmospheric profiles consist of the 6 US76 standard atmospheric profiles:
#   TROPICAL
#   MIDLATITUDE SUMMER
#   MIDLATITUDE WINTER
#   SUBARCTIC SUMMER
#   SUBARCTIC WINTER
#   U. S. STANDARD,  1976
#
# These form the basis for the test suite profile datasets: the number of
# profiles defined here is the number of profiles per test.
#
# NB The CO2 and N2O profiles have been modified from the originals to
#    be more like present-day values.
#
# There are also cloud liquid water, cloud+cfrac, and aerosol profiles.
# These are not based on any physical measurements and do not relate to
# the US76 profiles in any particular way. These must contain the same
# number of profiles as the basis set defined above.
#
# Following this the remaining RTTOV profile parameters are defined for
# each of the above profiles: these are also not based on measurements,
# but are designed to exercise different parts of RTTOV so they too do
# not correspond directly to the US76 profiles.
#
# Finally this file contains standard RTTOV coefficient pressure levels
# and RTTOV reference trace gas profiles on those levels.
#
# If you want to have more than 6 profiles per test dataset then you need
# to add the additional profile data to every array for each extra profile.
# The profile-generation code should handle everything else.
#
# You can also add other RTTOV pressure level/trace gas sets (e.g. 44L)
# and it is then very easy to use the profile generation code to write
# out additional datasets based on these levels.
#
# If you find a test fails for a particular profile the data below may be
# helpful in identifying how that profile differs from the others which may
# be useful in debugging the code (e.g. is the surface pressure very low/high,
# does the profile contain non-zero cfraction, etc)
#
# ============================================================================

import numpy as np

#-----------------------------------------------------------------------------
# US76 atmospheric profiles
#-----------------------------------------------------------------------------

#     PRESSURE /
p = np.array([[1.013E+03 , 9.040E+02 , 8.050E+02 , 7.150E+02 , 6.330E+02 ,
             5.590E+02 , 4.920E+02 , 4.320E+02 , 3.780E+02 , 3.290E+02 ,
             2.860E+02 , 2.470E+02 , 2.130E+02 , 1.820E+02 , 1.560E+02 ,
             1.320E+02 , 1.110E+02 , 9.370E+01 , 7.890E+01 , 6.660E+01 ,
             5.650E+01 , 4.800E+01 , 4.090E+01 , 3.500E+01 , 3.000E+01 ,
             2.570E+01 , 1.763E+01 , 1.220E+01 , 8.520E+00 , 6.000E+00 ,
             4.260E+00 , 3.050E+00 , 2.200E+00 , 1.590E+00 , 1.160E+00 ,
             8.540E-01 , 4.560E-01 , 2.390E-01 , 1.210E-01 , 5.800E-02 ,
             2.600E-02 , 1.100E-02 , 4.400E-03 , 1.720E-03 , 6.880E-04 ,
             2.890E-04 , 1.300E-04 , 6.470E-05 , 3.600E-05 , 2.250E-05 ],
             [1.013E+03 , 9.020E+02 , 8.020E+02 , 7.100E+02 , 6.280E+02 ,
             5.540E+02 , 4.870E+02 , 4.260E+02 , 3.720E+02 , 3.240E+02 ,
             2.810E+02 , 2.430E+02 , 2.090E+02 , 1.790E+02 , 1.530E+02 ,
             1.300E+02 , 1.110E+02 , 9.500E+01 , 8.120E+01 , 6.950E+01 ,
             5.950E+01 , 5.100E+01 , 4.370E+01 , 3.760E+01 , 3.220E+01 ,
             2.770E+01 , 1.907E+01 , 1.320E+01 , 9.300E+00 , 6.520E+00 ,
             4.640E+00 , 3.330E+00 , 2.410E+00 , 1.760E+00 , 1.290E+00 ,
             9.510E-01 , 5.150E-01 , 2.720E-01 , 1.390E-01 , 6.700E-02 ,
             3.000E-02 , 1.200E-02 , 4.480E-03 , 1.640E-03 , 6.250E-04 ,
             2.580E-04 , 1.170E-04 , 6.110E-05 , 3.560E-05 , 2.270E-05 ],
             [1.018E+03 , 8.973E+02 , 7.897E+02 , 6.938E+02 , 6.081E+02 ,
             5.313E+02 , 4.627E+02 , 4.016E+02 , 3.473E+02 , 2.993E+02 ,
             2.568E+02 , 2.199E+02 , 1.882E+02 , 1.611E+02 , 1.378E+02 ,
             1.178E+02 , 1.007E+02 , 8.610E+01 , 7.360E+01 , 6.280E+01 ,
             5.370E+01 , 4.580E+01 , 3.910E+01 , 3.340E+01 , 2.860E+01 ,
             2.440E+01 , 1.646E+01 , 1.110E+01 , 7.560E+00 , 5.180E+00 ,
             3.600E+00 , 2.530E+00 , 1.800E+00 , 1.290E+00 , 9.400E-01 ,
             6.830E-01 , 3.620E-01 , 1.880E-01 , 9.500E-02 , 4.700E-02 ,
             2.220E-02 , 1.030E-02 , 4.560E-03 , 1.980E-03 , 8.770E-04 ,
             4.074E-04 , 2.000E-04 , 1.057E-04 , 5.980E-05 , 3.600E-05 ],
             [1.010E+03 , 8.960E+02 , 7.929E+02 , 7.000E+02 , 6.160E+02 ,
             5.410E+02 , 4.740E+02 , 4.130E+02 , 3.590E+02 , 3.108E+02 ,
             2.677E+02 , 2.300E+02 , 1.977E+02 , 1.700E+02 , 1.460E+02 ,
             1.260E+02 , 1.080E+02 , 9.280E+01 , 7.980E+01 , 6.860E+01 ,
             5.900E+01 , 5.070E+01 , 4.360E+01 , 3.750E+01 , 3.228E+01 ,
             2.780E+01 , 1.923E+01 , 1.340E+01 , 9.400E+00 , 6.610E+00 ,
             4.720E+00 , 3.400E+00 , 2.480E+00 , 1.820E+00 , 1.340E+00 ,
             9.870E-01 , 5.370E-01 , 2.880E-01 , 1.470E-01 , 7.100E-02 ,
             3.200E-02 , 1.250E-02 , 4.510E-03 , 1.610E-03 , 6.060E-04 ,
             2.480E-04 , 1.130E-04 , 6.000E-05 , 3.540E-05 , 2.260E-05 ],
             [1.013E+03 , 8.878E+02 , 7.775E+02 , 6.798E+02 , 5.932E+02 ,
             5.158E+02 , 4.467E+02 , 3.853E+02 , 3.308E+02 , 2.829E+02 ,
             2.418E+02 , 2.067E+02 , 1.766E+02 , 1.510E+02 , 1.291E+02 ,
             1.103E+02 , 9.431E+01 , 8.058E+01 , 6.882E+01 , 5.875E+01 ,
             5.014E+01 , 4.277E+01 , 3.647E+01 , 3.109E+01 , 2.649E+01 ,
             2.256E+01 , 1.513E+01 , 1.020E+01 , 6.910E+00 , 4.701E+00 ,
             3.230E+00 , 2.243E+00 , 1.570E+00 , 1.113E+00 , 7.900E-01 ,
             5.719E-01 , 2.990E-01 , 1.550E-01 , 7.900E-02 , 4.000E-02 ,
             2.000E-02 , 9.660E-03 , 4.500E-03 , 2.022E-03 , 9.070E-04 ,
             4.230E-04 , 2.070E-04 , 1.080E-04 , 6.000E-05 , 3.590E-05 ],
             [1.013E+03 , 8.988E+02 , 7.950E+02 , 7.012E+02 , 6.166E+02 ,
             5.405E+02 , 4.722E+02 , 4.111E+02 , 3.565E+02 , 3.080E+02 ,
             2.650E+02 , 2.270E+02 , 1.940E+02 , 1.658E+02 , 1.417E+02 ,
             1.211E+02 , 1.035E+02 , 8.850E+01 , 7.565E+01 , 6.467E+01 ,
             5.529E+01 , 4.729E+01 , 4.047E+01 , 3.467E+01 , 2.972E+01 ,
             2.549E+01 , 1.743E+01 , 1.197E+01 , 8.258E+00 , 5.746E+00 ,
             4.041E+00 , 2.871E+00 , 2.060E+00 , 1.491E+00 , 1.090E+00 ,
             7.978E-01 , 4.250E-01 , 2.190E-01 , 1.090E-01 , 5.220E-02 ,
             2.400E-02 , 1.050E-02 , 4.460E-03 , 1.840E-03 , 7.600E-04 ,
             3.200E-04 , 1.450E-04 , 7.100E-05 , 4.010E-05 , 2.540E-05 ]])

#       TEMPERATURE /
t = np.array([[299.70 ,    293.70 ,    287.70 ,    283.70 ,    277.00 ,
              270.30 ,    263.60 ,    257.00 ,    250.30 ,    243.60 ,
              237.00 ,    230.10 ,    223.60 ,    217.00 ,    210.30 ,
              203.70 ,    197.00 ,    194.80 ,    198.80 ,    202.70 ,
              206.70 ,    210.70 ,    214.60 ,    217.00 ,    219.20 ,
              221.40 ,    227.00 ,    232.30 ,    237.70 ,    243.10 ,
              248.50 ,    254.00 ,    259.40 ,    264.80 ,    269.60 ,
              270.20 ,    263.40 ,    253.10 ,    236.00 ,    218.90 ,
              201.80 ,    184.80 ,    177.10 ,    177.00 ,    184.30 ,
              190.70 ,    212.00 ,    241.60 ,    299.70 ,    380.00],
              [294.20 ,    289.70 ,    285.20 ,    279.20 ,    273.20 ,
              267.20 ,    261.20 ,    254.70 ,    248.20 ,    241.70 ,
              235.30 ,    228.80 ,    222.30 ,    215.80 ,    215.70 ,
              215.70 ,    215.70 ,    215.70 ,    216.80 ,    217.90 ,
              219.20 ,    220.40 ,    221.60 ,    222.80 ,    223.90 ,
              225.10 ,    228.45 ,    233.70 ,    239.00 ,    245.20 ,
              251.30 ,    257.50 ,    263.70 ,    269.90 ,    275.20 ,
              275.70 ,    269.30 ,    257.10 ,    240.10 ,    218.10 ,
              196.10 ,    174.10 ,    165.10 ,    165.00 ,    178.30 ,
              190.50 ,    222.20 ,    262.40 ,    316.80 ,    380.00],
              [272.20 ,    268.70 ,    265.20 ,    261.70 ,    255.70 ,
              249.70 ,    243.70 ,    237.70 ,    231.70 ,    225.70 ,
              219.70 ,    219.20 ,    218.70 ,    218.20 ,    217.70 ,
              217.20 ,    216.70 ,    216.20 ,    215.70 ,    215.20 ,
              215.20 ,    215.20 ,    215.20 ,    215.20 ,    215.20 ,
              215.20 ,    215.50 ,    217.40 ,    220.40 ,    227.90 ,
              235.50 ,    243.20 ,    250.80 ,    258.50 ,    265.10 ,
              265.70 ,    260.60 ,    250.80 ,    240.90 ,    230.70 ,
              220.40 ,    210.10 ,    199.80 ,    199.50 ,    208.30 ,
              218.60 ,    237.10 ,    259.50 ,    293.00 ,    333.00],
              [287.20 ,    281.70 ,    276.30 ,    270.90 ,    265.50 ,
              260.10 ,    253.10 ,    246.10 ,    239.20 ,    232.20 ,
              225.20 ,    225.20 ,    225.20 ,    225.20 ,    225.20 ,
              225.20 ,    225.20 ,    225.20 ,    225.20 ,    225.20 ,
              225.20 ,    225.20 ,    225.20 ,    225.20 ,    226.60 ,
              228.10 ,    231.00 ,    235.10 ,    240.00 ,    247.20 ,
              254.60 ,    262.10 ,    269.50 ,    273.60 ,    276.20 ,
              277.20 ,    274.00 ,    262.70 ,    239.70 ,    216.60 ,
              193.60 ,    170.60 ,    161.70 ,    161.60 ,    176.80 ,
              190.40 ,    226.00 ,    270.10 ,    322.70 ,    380.00],
              [257.20 ,    259.10 ,    255.90 ,    252.70 ,    247.70 ,
              240.90 ,    234.10 ,    227.30 ,    220.60 ,    217.20 ,
              217.20 ,    217.20 ,    217.20 ,    217.20 ,    217.20 ,
              217.20 ,    216.60 ,    216.00 ,    215.40 ,    214.80 ,
              214.20 ,    213.60 ,    213.00 ,    212.40 ,    211.80 ,
              211.20 ,    213.60 ,    216.00 ,    218.50 ,    222.30 ,
              228.50 ,    234.70 ,    240.80 ,    247.00 ,    253.20 ,
              259.30 ,    259.10 ,    250.90 ,    248.40 ,    245.40 ,
              234.70 ,    223.90 ,    213.10 ,    202.30 ,    211.00 ,
              218.50 ,    234.00 ,    252.60 ,    288.50 ,    333.00],
              [288.20 ,    281.70 ,    275.20 ,    268.70 ,    262.20 ,
              255.70 ,    249.20 ,    242.70 ,    236.20 ,    229.70 ,
              223.30 ,    216.80 ,    216.70 ,    216.70 ,    216.70 ,
              216.70 ,    216.70 ,    216.70 ,    216.70 ,    216.70 ,
              216.70 ,    217.60 ,    218.60 ,    219.60 ,    220.60 ,
              221.60 ,    224.00 ,    226.50 ,    229.60 ,    236.50 ,
              243.40 ,    250.40 ,    257.30 ,    264.20 ,    270.60 ,
              270.70 ,    260.80 ,    247.00 ,    233.30 ,    219.60 ,
              208.40 ,    198.60 ,    188.90 ,    186.90 ,    188.40 ,
              195.10 ,    208.80 ,    240.00 ,    300.00 ,    360.00]])

#        H2O /
q = np.array([[2.593E+04 , 1.949E+04 , 1.534E+04 , 8.600E+03 , 4.441E+03 ,
             3.346E+03 , 2.101E+03 , 1.289E+03 , 7.637E+02 , 4.098E+02 ,
             1.912E+02 , 7.306E+01 , 2.905E+01 , 9.900E+00 , 6.220E+00 ,
             4.000E+00 , 3.000E+00 , 2.900E+00 , 2.750E+00 , 2.600E+00 ,
             2.600E+00 , 2.650E+00 , 2.800E+00 , 2.900E+00 , 3.200E+00 ,
             3.250E+00 , 3.600E+00 , 4.000E+00 , 4.300E+00 , 4.600E+00 ,
             4.900E+00 , 5.200E+00 , 5.500E+00 , 5.700E+00 , 5.900E+00 ,
             6.000E+00 , 6.000E+00 , 6.000E+00 , 5.400E+00 , 4.500E+00 ,
             3.300E+00 , 2.100E+00 , 1.300E+00 , 8.500E-01 , 5.400E-01 ,
             4.000E-01 , 3.400E-01 , 2.800E-01 , 2.400E-01 , 2.000E-01],
             [1.876E+04 , 1.378E+04 , 9.680E+03 , 5.984E+03 , 3.813E+03 ,
             2.225E+03 , 1.510E+03 , 1.020E+03 , 6.464E+02 , 4.129E+02 ,
             2.472E+02 , 9.556E+01 , 2.944E+01 , 8.000E+00 , 5.000E+00 ,
             3.400E+00 , 3.300E+00 , 3.200E+00 , 3.150E+00 , 3.200E+00 ,
             3.300E+00 , 3.450E+00 , 3.600E+00 , 3.850E+00 , 4.000E+00 ,
             4.200E+00 , 4.450E+00 , 4.700E+00 , 4.850E+00 , 4.950E+00 ,
             5.000E+00 , 5.100E+00 , 5.300E+00 , 5.450E+00 , 5.500E+00 ,
             5.500E+00 , 5.350E+00 , 5.000E+00 , 4.400E+00 , 3.700E+00 ,
             2.950E+00 , 2.100E+00 , 1.330E+00 , 8.500E-01 , 5.400E-01 ,
             4.000E-01 , 3.400E-01 , 2.800E-01 , 2.400E-01 , 2.000E-01],
             [4.316E+03 , 3.454E+03 , 2.788E+03 , 2.088E+03 , 1.280E+03 ,
             8.241E+02 , 5.103E+02 , 2.321E+02 , 1.077E+02 , 5.566E+01 ,
             2.960E+01 , 1.000E+01 , 6.000E+00 , 5.000E+00 , 4.800E+00 ,
             4.700E+00 , 4.600E+00 , 4.500E+00 , 4.500E+00 , 4.500E+00 ,
             4.500E+00 , 4.500E+00 , 4.530E+00 , 4.550E+00 , 4.600E+00 ,
             4.650E+00 , 4.700E+00 , 4.750E+00 , 4.800E+00 , 4.850E+00 ,
             4.900E+00 , 4.950E+00 , 5.000E+00 , 5.000E+00 , 5.000E+00 ,
             4.950E+00 , 4.850E+00 , 4.500E+00 , 4.000E+00 , 3.300E+00 ,
             2.700E+00 , 2.000E+00 , 1.330E+00 , 8.500E-01 , 5.400E-01 ,
             4.000E-01 , 3.400E-01 , 2.800E-01 , 2.400E-01 , 2.000E-01],
             [1.194E+04 , 8.701E+03 , 6.750E+03 , 4.820E+03 , 3.380E+03 ,
             2.218E+03 , 1.330E+03 , 7.971E+02 , 3.996E+02 , 1.300E+02 ,
             4.240E+01 , 1.330E+01 , 6.000E+00 , 4.450E+00 , 4.000E+00 ,
             4.000E+00 , 4.000E+00 , 4.050E+00 , 4.300E+00 , 4.500E+00 ,
             4.600E+00 , 4.700E+00 , 4.800E+00 , 4.830E+00 , 4.850E+00 ,
             4.900E+00 , 4.950E+00 , 5.000E+00 , 5.000E+00 , 5.000E+00 ,
             5.000E+00 , 5.000E+00 , 5.000E+00 , 5.000E+00 , 5.000E+00 ,
             4.950E+00 , 4.850E+00 , 4.500E+00 , 4.000E+00 , 3.300E+00 ,
             2.700E+00 , 2.000E+00 , 1.330E+00 , 8.500E-01 , 5.400E-01 ,
             4.000E-01 , 3.400E-01 , 2.800E-01 , 2.400E-01 , 2.000E-01],
             [1.405E+03 , 1.615E+03 , 1.427E+03 , 1.166E+03 , 7.898E+02 ,
             4.309E+02 , 2.369E+02 , 1.470E+02 , 3.384E+01 , 2.976E+01 ,
             2.000E+01 , 1.000E+01 , 6.000E+00 , 4.450E+00 , 4.500E+00 ,
             4.550E+00 , 4.600E+00 , 4.650E+00 , 4.700E+00 , 4.750E+00 ,
             4.800E+00 , 4.850E+00 , 4.900E+00 , 4.950E+00 , 5.000E+00 ,
             5.000E+00 , 5.000E+00 , 5.000E+00 , 5.000E+00 , 5.000E+00 ,
             5.000E+00 , 5.000E+00 , 5.000E+00 , 5.000E+00 , 5.000E+00 ,
             4.950E+00 , 4.850E+00 , 4.500E+00 , 4.000E+00 , 3.300E+00 ,
             2.700E+00 , 2.000E+00 , 1.330E+00 , 8.500E-01 , 5.400E-01 ,
             4.000E-01 , 3.400E-01 , 2.800E-01 , 2.400E-01 , 2.000E-01],
             [7.745E+03 , 6.071E+03 , 4.631E+03 , 3.182E+03 , 2.158E+03 ,
             1.397E+03 , 9.254E+02 , 5.720E+02 , 3.667E+02 , 1.583E+02 ,
             6.996E+01 , 3.613E+01 , 1.906E+01 , 1.085E+01 , 5.927E+00 ,
             5.000E+00 , 3.950E+00 , 3.850E+00 , 3.825E+00 , 3.850E+00 ,
             3.900E+00 , 3.975E+00 , 4.065E+00 , 4.200E+00 , 4.300E+00 ,
             4.425E+00 , 4.575E+00 , 4.725E+00 , 4.825E+00 , 4.900E+00 ,
             4.950E+00 , 5.025E+00 , 5.150E+00 , 5.225E+00 , 5.250E+00 ,
             5.225E+00 , 5.100E+00 , 4.750E+00 , 4.200E+00 , 3.500E+00 ,
             2.825E+00 , 2.050E+00 , 1.330E+00 , 8.500E-01 , 5.400E-01 ,
             4.000E-01 , 3.400E-01 , 2.800E-01 , 2.400E-01 , 2.000E-01]])

#       CO2 /  - US76 original profile has been replaced by more contemporary values
#co2 = np.array([[3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.280E+02 , 3.200E+02 , 3.100E+02 , 2.700E+02 ,
             #1.950E+02 , 1.100E+02 , 6.000E+01 , 4.000E+01 , 3.500E+01],
             #[3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.280E+02 , 3.200E+02 , 3.100E+02 , 2.700E+02 ,
             #1.950E+02 , 1.100E+02 , 6.000E+01 , 4.000E+01 , 3.500E+01],
             #[3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.280E+02 , 3.200E+02 , 3.100E+02 , 2.700E+02 ,
             #1.950E+02 , 1.100E+02 , 6.000E+01 , 4.000E+01 , 3.500E+01],
             #[3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.280E+02 , 3.200E+02 , 3.100E+02 , 2.700E+02 ,
             #1.950E+02 , 1.100E+02 , 6.000E+01 , 4.000E+01 , 3.500E+01],
             #[3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.280E+02 , 3.200E+02 , 3.100E+02 , 2.700E+02 ,
             #1.950E+02 , 1.100E+02 , 6.000E+01 , 4.000E+01 , 3.500E+01],
             #[3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 , 3.300E+02 ,
             #3.300E+02 , 3.280E+02 , 3.200E+02 , 3.100E+02 , 2.700E+02 ,
             #1.950E+02 , 1.100E+02 , 6.000E+01 , 4.000E+01 , 3.500E+01]])

co2 = np.array([[4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 3.980E+02 , 3.950E+02 , 3.940E+02 , 3.910E+02 ,
             3.850E+02 , 3.840E+02 , 3.830E+02 , 3.815E+02 , 3.800E+02],
             [4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 3.980E+02 , 3.950E+02 , 3.940E+02 , 3.910E+02 ,
             3.850E+02 , 3.840E+02 , 3.830E+02 , 3.815E+02 , 3.800E+02],
             [4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 3.980E+02 , 3.950E+02 , 3.940E+02 , 3.910E+02 ,
             3.850E+02 , 3.840E+02 , 3.830E+02 , 3.815E+02 , 3.800E+02],
             [4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 3.980E+02 , 3.950E+02 , 3.940E+02 , 3.910E+02 ,
             3.850E+02 , 3.840E+02 , 3.830E+02 , 3.815E+02 , 3.800E+02],
             [4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 3.980E+02 , 3.950E+02 , 3.940E+02 , 3.910E+02 ,
             3.850E+02 , 3.840E+02 , 3.830E+02 , 3.815E+02 , 3.800E+02],
             [4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 , 4.000E+02 ,
             4.000E+02 , 3.980E+02 , 3.950E+02 , 3.940E+02 , 3.910E+02 ,
             3.850E+02 , 3.840E+02 , 3.830E+02 , 3.815E+02 , 3.800E+02]])

#       OZONE /
o3 = np.array([[2.869E-02 , 3.150E-02 , 3.342E-02 , 3.504E-02 , 3.561E-02 ,
             3.767E-02 , 3.989E-02 , 4.223E-02 , 4.471E-02 , 5.000E-02 ,
             5.595E-02 , 6.613E-02 , 7.815E-02 , 9.289E-02 , 1.050E-01 ,
             1.256E-01 , 1.444E-01 , 2.500E-01 , 5.000E-01 , 9.500E-01 ,
             1.400E+00 , 1.800E+00 , 2.400E+00 , 3.400E+00 , 4.300E+00 ,
             5.400E+00 , 7.800E+00 , 9.300E+00 , 9.850E+00 , 9.700E+00 ,
             8.800E+00 , 7.500E+00 , 5.900E+00 , 4.500E+00 , 3.450E+00 ,
             2.800E+00 , 1.800E+00 , 1.100E+00 , 6.500E-01 , 3.000E-01 ,
             1.800E-01 , 3.300E-01 , 5.000E-01 , 5.200E-01 , 5.000E-01 ,
             4.000E-01 , 2.000E-01 , 5.000E-02 , 5.000E-03 , 5.000E-04],
             [3.017E-02 , 3.337E-02 , 3.694E-02 , 4.222E-02 , 4.821E-02 ,
             5.512E-02 , 6.408E-02 , 7.764E-02 , 9.126E-02 , 1.111E-01 ,
             1.304E-01 , 1.793E-01 , 2.230E-01 , 3.000E-01 , 4.400E-01 ,
             5.000E-01 , 6.000E-01 , 7.000E-01 , 1.000E+00 , 1.500E+00 ,
             2.000E+00 , 2.400E+00 , 2.900E+00 , 3.400E+00 , 4.000E+00 ,
             4.800E+00 , 6.000E+00 , 7.000E+00 , 8.100E+00 , 8.900E+00 ,
             8.700E+00 , 7.550E+00 , 5.900E+00 , 4.500E+00 , 3.500E+00 ,
             2.800E+00 , 1.800E+00 , 1.300E+00 , 8.000E-01 , 4.000E-01 ,
             1.900E-01 , 2.000E-01 , 5.700E-01 , 7.500E-01 , 7.000E-01 ,
             4.000E-01 , 2.000E-01 , 5.000E-02 , 5.000E-03 , 5.000E-04],
             [2.778E-02 , 2.800E-02 , 2.849E-02 , 3.200E-02 , 3.567E-02 ,
             4.720E-02 , 5.837E-02 , 7.891E-02 , 1.039E-01 , 1.567E-01 ,
             2.370E-01 , 3.624E-01 , 5.232E-01 , 7.036E-01 , 8.000E-01 ,
             9.000E-01 , 1.100E+00 , 1.400E+00 , 1.800E+00 , 2.300E+00 ,
             2.900E+00 , 3.500E+00 , 3.900E+00 , 4.300E+00 , 4.700E+00 ,
             5.100E+00 , 5.600E+00 , 6.100E+00 , 6.800E+00 , 7.100E+00 ,
             7.200E+00 , 6.900E+00 , 5.900E+00 , 4.600E+00 , 3.700E+00 ,
             2.750E+00 , 1.700E+00 , 1.000E-00 , 5.500E-01 , 3.200E-01 ,
             2.500E-01 , 2.300E-01 , 5.500E-01 , 8.000E-01 , 8.000E-01 ,
             4.000E-01 , 2.000E-01 , 5.000E-02 , 5.000E-03 , 5.000E-04],
             [2.412E-02 , 2.940E-02 , 3.379E-02 , 3.887E-02 , 4.478E-02 ,
             5.328E-02 , 6.564E-02 , 7.738E-02 , 9.114E-02 , 1.420E-01 ,
             1.890E-01 , 3.050E-01 , 4.100E-01 , 5.000E-01 , 6.000E-01 ,
             7.000E-01 , 8.500E-01 , 1.000E+00 , 1.300E+00 , 1.700E+00 ,
             2.100E+00 , 2.700E+00 , 3.300E+00 , 3.700E+00 , 4.200E+00 ,
             4.500E+00 , 5.300E+00 , 5.700E+00 , 6.900E+00 , 7.700E+00 ,
             7.800E+00 , 7.000E+00 , 5.400E+00 , 4.200E+00 , 3.200E+00 ,
             2.500E+00 , 1.700E+00 , 1.200E+00 , 8.000E-01 , 4.000E-01 ,
             2.000E-01 , 1.800E-01 , 6.500E-01 , 9.000E-01 , 8.000E-01 ,
             4.000E-01 , 2.000E-01 , 5.000E-02 , 5.000E-03 , 5.000E-04],
             [1.802E-02 , 2.072E-02 , 2.336E-02 , 2.767E-02 , 3.253E-02 ,
             3.801E-02 , 4.446E-02 , 7.252E-02 , 1.040E-01 , 2.100E-01 ,
             3.000E-01 , 3.500E-01 , 4.000E-01 , 6.500E-01 , 9.000E-01 ,
             1.200E+00 , 1.500E+00 , 1.900E+00 , 2.450E+00 , 3.100E+00 ,
             3.700E+00 , 4.000E+00 , 4.200E+00 , 4.500E+00 , 4.600E+00 ,
             4.700E+00 , 4.900E+00 , 5.400E+00 , 5.900E+00 , 6.200E+00 ,
             6.250E+00 , 5.900E+00 , 5.100E+00 , 4.100E+00 , 3.000E+00 ,
             2.600E+00 , 1.600E+00 , 9.500E-01 , 6.500E-01 , 5.000E-01 ,
             3.300E-01 , 1.300E-01 , 7.500E-01 , 8.000E-01 , 8.000E-01 ,
             4.000E-01 , 2.000E-01 , 5.000E-02 , 5.000E-03 , 5.000E-04],
             [2.660E-02 , 2.931E-02 , 3.237E-02 , 3.318E-02 , 3.387E-02 ,
             3.768E-02 , 4.112E-02 , 5.009E-02 , 5.966E-02 , 9.168E-02 ,
             1.313E-01 , 2.149E-01 , 3.095E-01 , 3.846E-01 , 5.030E-01 ,
             6.505E-01 , 8.701E-01 , 1.187E+00 , 1.587E+00 , 2.030E+00 ,
             2.579E+00 , 3.028E+00 , 3.647E+00 , 4.168E+00 , 4.627E+00 ,
             5.118E+00 , 5.803E+00 , 6.553E+00 , 7.373E+00 , 7.837E+00 ,
             7.800E+00 , 7.300E+00 , 6.200E+00 , 5.250E+00 , 4.100E+00 ,
             3.100E+00 , 1.800E+00 , 1.100E+00 , 7.000E-01 , 3.000E-01 ,
             2.500E-01 , 3.000E-01 , 5.000E-01 , 7.000E-01 , 7.000E-01 ,
             4.000E-01 , 2.000E-01 , 5.000E-02 , 5.000E-03 , 5.000E-04]])

#        N2O /
n2o = np.array([[3.200E-01 , 3.200E-01 , 3.200E-01 , 3.200E-01 , 3.200E-01 ,
             3.200E-01 , 3.200E-01 , 3.200E-01 , 3.200E-01 , 3.195E-01 ,
             3.179E-01 , 3.140E-01 , 3.095E-01 , 3.048E-01 , 2.999E-01 ,
             2.944E-01 , 2.877E-01 , 2.783E-01 , 2.671E-01 , 2.527E-01 ,
             2.365E-01 , 2.194E-01 , 2.051E-01 , 1.967E-01 , 1.875E-01 ,
             1.756E-01 , 1.588E-01 , 1.416E-01 , 1.165E-01 , 9.275E-02 ,
             6.693E-02 , 4.513E-02 , 2.751E-02 , 1.591E-02 , 9.378E-03 ,
             4.752E-03 , 3.000E-03 , 2.065E-03 , 1.507E-03 , 1.149E-03 ,
             8.890E-04 , 7.056E-04 , 5.716E-04 , 4.708E-04 , 3.932E-04 ,
             3.323E-04 , 2.837E-04 , 2.443E-04 , 2.120E-04 , 1.851E-04],
             [3.200E-01 , 3.200E-01 , 3.200E-01 , 3.200E-01 , 3.200E-01 ,
             3.200E-01 , 3.200E-01 , 3.200E-01 , 3.195E-01 , 3.163E-01 ,
             3.096E-01 , 2.989E-01 , 2.936E-01 , 2.860E-01 , 2.800E-01 ,
             2.724E-01 , 2.611E-01 , 2.421E-01 , 2.174E-01 , 1.843E-01 ,
             1.607E-01 , 1.323E-01 , 1.146E-01 , 1.035E-01 , 9.622E-02 ,
             8.958E-02 , 8.006E-02 , 6.698E-02 , 4.958E-02 , 3.695E-02 ,
             2.519E-02 , 1.736E-02 , 1.158E-02 , 7.665E-03 , 5.321E-03 ,
             3.215E-03 , 2.030E-03 , 1.397E-03 , 1.020E-03 , 7.772E-04 ,
             6.257E-04 , 5.166E-04 , 4.352E-04 , 3.727E-04 , 3.237E-04 ,
             2.844E-04 , 2.524E-04 , 2.260E-04 , 2.039E-04 , 1.851E-04],
             [3.200E-01 , 3.200E-01 , 3.200E-01 , 3.200E-01 , 3.200E-01 ,
             3.200E-01 , 3.200E-01 , 3.200E-01 , 3.195E-01 , 3.163E-01 ,
             3.096E-01 , 2.989E-01 , 2.936E-01 , 2.860E-01 , 2.800E-01 ,
             2.724E-01 , 2.611E-01 , 2.421E-01 , 2.174E-01 , 1.843E-01 ,
             1.621E-01 , 1.362E-01 , 1.230E-01 , 1.124E-01 , 1.048E-01 ,
             9.661E-02 , 8.693E-02 , 7.524E-02 , 6.126E-02 , 5.116E-02 ,
             3.968E-02 , 2.995E-02 , 2.080E-02 , 1.311E-02 , 8.071E-03 ,
             4.164E-03 , 2.629E-03 , 1.809E-03 , 1.321E-03 , 1.007E-03 ,
             7.883E-04 , 6.333E-04 , 5.194E-04 , 4.333E-04 , 3.666E-04 ,
             3.140E-04 , 2.717E-04 , 2.373E-04 , 2.089E-04 , 1.851E-04],
             [3.100E-01 , 3.100E-01 , 3.100E-01 , 3.100E-01 , 3.079E-01 ,
             3.024E-01 , 2.906E-01 , 2.822E-01 , 2.759E-01 , 2.703E-01 ,
             2.651E-01 , 2.600E-01 , 2.549E-01 , 2.494E-01 , 2.433E-01 ,
             2.355E-01 , 2.282E-01 , 2.179E-01 , 2.035E-01 , 1.817E-01 ,
             1.567E-01 , 1.350E-01 , 1.218E-01 , 1.102E-01 , 9.893E-02 ,
             8.775E-02 , 7.327E-02 , 5.941E-02 , 4.154E-02 , 3.032E-02 ,
             1.949E-02 , 1.274E-02 , 9.001E-03 , 6.286E-03 , 4.558E-03 ,
             2.795E-03 , 1.765E-03 , 1.214E-03 , 8.866E-04 , 6.756E-04 ,
             5.538E-04 , 4.649E-04 , 3.979E-04 , 3.459E-04 , 3.047E-04 ,
             2.713E-04 , 2.439E-04 , 2.210E-04 , 2.017E-04 , 1.851E-04],
             [3.200E-01 , 3.200E-01 , 3.200E-01 , 3.200E-01 , 3.200E-01 ,
             3.200E-01 , 3.200E-01 , 3.200E-01 , 3.195E-01 , 3.163E-01 ,
             3.096E-01 , 2.989E-01 , 2.936E-01 , 2.860E-01 , 2.800E-01 ,
             2.724E-01 , 2.611E-01 , 2.421E-01 , 2.174E-01 , 1.843E-01 ,
             1.621E-01 , 1.362E-01 , 1.230E-01 , 1.122E-01 , 1.043E-01 ,
             9.570E-02 , 8.598E-02 , 7.314E-02 , 5.710E-02 , 4.670E-02 ,
             3.439E-02 , 2.471E-02 , 1.631E-02 , 1.066E-02 , 7.064E-03 ,
             3.972E-03 , 2.508E-03 , 1.726E-03 , 1.260E-03 , 9.602E-04 ,
             7.554E-04 , 6.097E-04 , 5.024E-04 , 4.210E-04 , 3.579E-04 ,
             3.080E-04 , 2.678E-04 , 2.350E-04 , 2.079E-04 , 1.851E-04],
             [3.200E-01 , 3.200E-01 , 3.200E-01 , 3.200E-01 , 3.200E-01 ,
             3.200E-01 , 3.200E-01 , 3.200E-01 , 3.200E-01 , 3.195E-01 ,
             3.179E-01 , 3.140E-01 , 3.095E-01 , 3.048E-01 , 2.999E-01 ,
             2.944E-01 , 2.877E-01 , 2.783E-01 , 2.671E-01 , 2.527E-01 ,
             2.365E-01 , 2.194E-01 , 2.051E-01 , 1.967E-01 , 1.875E-01 ,
             1.756E-01 , 1.588E-01 , 1.416E-01 , 1.165E-01 , 9.275E-02 ,
             6.693E-02 , 4.513E-02 , 2.751E-02 , 1.591E-02 , 9.378E-03 ,
             4.752E-03 , 3.000E-03 , 2.065E-03 , 1.507E-03 , 1.149E-03 ,
             8.890E-04 , 7.056E-04 , 5.716E-04 , 4.708E-04 , 3.932E-04 ,
             3.323E-04 , 2.837E-04 , 2.443E-04 , 2.120E-04 , 1.851E-04]])

#       CO /
co = np.array([[1.500E-01 , 1.450E-01 , 1.399E-01 , 1.349E-01 , 1.312E-01 ,
             1.303E-01 , 1.288E-01 , 1.247E-01 , 1.185E-01 , 1.094E-01 ,
             9.962E-02 , 8.964E-02 , 7.814E-02 , 6.374E-02 , 5.025E-02 ,
             3.941E-02 , 3.069E-02 , 2.489E-02 , 1.966E-02 , 1.549E-02 ,
             1.331E-02 , 1.232E-02 , 1.232E-02 , 1.307E-02 , 1.400E-02 ,
             1.521E-02 , 1.722E-02 , 1.995E-02 , 2.266E-02 , 2.487E-02 ,
             2.738E-02 , 3.098E-02 , 3.510E-02 , 3.987E-02 , 4.482E-02 ,
             5.092E-02 , 5.985E-02 , 6.960E-02 , 9.188E-02 , 1.938E-01 ,
             5.688E-01 , 1.549E+00 , 3.849E+00 , 6.590E+00 , 1.044E+01 ,
             1.705E+01 , 2.471E+01 , 3.358E+01 , 4.148E+01 , 5.000E+01],
             [1.500E-01 , 1.450E-01 , 1.399E-01 , 1.349E-01 , 1.312E-01 ,
             1.303E-01 , 1.288E-01 , 1.247E-01 , 1.185E-01 , 1.094E-01 ,
             9.962E-02 , 8.964E-02 , 7.814E-02 , 6.374E-02 , 5.025E-02 ,
             3.941E-02 , 3.069E-02 , 2.489E-02 , 1.966E-02 , 1.549E-02 ,
             1.331E-02 , 1.232E-02 , 1.232E-02 , 1.307E-02 , 1.400E-02 ,
             1.521E-02 , 1.722E-02 , 1.995E-02 , 2.266E-02 , 2.487E-02 ,
             2.716E-02 , 2.962E-02 , 3.138E-02 , 3.307E-02 , 3.487E-02 ,
             3.645E-02 , 3.923E-02 , 4.673E-02 , 6.404E-02 , 1.177E-01 ,
             2.935E-01 , 6.815E-01 , 1.465E+00 , 2.849E+00 , 5.166E+00 ,
             1.008E+01 , 1.865E+01 , 2.863E+01 , 3.890E+01 , 5.000E+01],
             [1.500E-01 , 1.450E-01 , 1.399E-01 , 1.349E-01 , 1.312E-01 ,
             1.303E-01 , 1.288E-01 , 1.247E-01 , 1.185E-01 , 1.094E-01 ,
             9.962E-02 , 8.964E-02 , 7.814E-02 , 6.374E-02 , 5.025E-02 ,
             3.941E-02 , 3.069E-02 , 2.489E-02 , 1.966E-02 , 1.549E-02 ,
             1.331E-02 , 1.232E-02 , 1.232E-02 , 1.307E-02 , 1.400E-02 ,
             1.498E-02 , 1.598E-02 , 1.710E-02 , 1.850E-02 , 1.997E-02 ,
             2.147E-02 , 2.331E-02 , 2.622E-02 , 3.057E-02 , 3.803E-02 ,
             6.245E-02 , 1.480E-01 , 2.926E-01 , 5.586E-01 , 1.078E+00 ,
             1.897E+00 , 2.960E+00 , 4.526E+00 , 6.862E+00 , 1.054E+01 ,
             1.709E+01 , 2.473E+01 , 3.359E+01 , 4.149E+01 , 5.000E+01],
             [1.500E-01 , 1.450E-01 , 1.399E-01 , 1.349E-01 , 1.312E-01 ,
             1.303E-01 , 1.288E-01 , 1.247E-01 , 1.185E-01 , 1.094E-01 ,
             9.962E-02 , 8.964E-02 , 7.814E-02 , 6.374E-02 , 5.025E-02 ,
             3.941E-02 , 3.069E-02 , 2.489E-02 , 1.966E-02 , 1.549E-02 ,
             1.331E-02 , 1.232E-02 , 1.232E-02 , 1.307E-02 , 1.400E-02 ,
             1.510E-02 , 1.649E-02 , 1.808E-02 , 1.997E-02 , 2.183E-02 ,
             2.343E-02 , 2.496E-02 , 2.647E-02 , 2.809E-02 , 2.999E-02 ,
             3.220E-02 , 3.650E-02 , 4.589E-02 , 6.375E-02 , 1.176E-01 ,
             3.033E-01 , 7.894E-01 , 1.823E+00 , 3.402E+00 , 5.916E+00 ,
             1.043E+01 , 1.881E+01 , 2.869E+01 , 3.892E+01 , 5.000E+01],
             [1.500E-01 , 1.450E-01 , 1.399E-01 , 1.349E-01 , 1.312E-01 ,
             1.303E-01 , 1.288E-01 , 1.247E-01 , 1.185E-01 , 1.094E-01 ,
             9.962E-02 , 8.964E-02 , 7.814E-02 , 6.374E-02 , 5.025E-02 ,
             3.941E-02 , 3.069E-02 , 2.489E-02 , 1.966E-02 , 1.549E-02 ,
             1.331E-02 , 1.232E-02 , 1.232E-02 , 1.307E-02 , 1.400E-02 ,
             1.521E-02 , 1.722E-02 , 2.037E-02 , 2.486E-02 , 3.168E-02 ,
             4.429E-02 , 6.472E-02 , 1.041E-01 , 1.507E-01 , 2.163E-01 ,
             3.141E-01 , 4.842E-01 , 7.147E-01 , 1.067E+00 , 1.516E+00 ,
             2.166E+00 , 3.060E+00 , 4.564E+00 , 6.877E+00 , 1.055E+01 ,
             1.710E+01 , 2.473E+01 , 3.359E+01 , 4.149E+01 , 5.000E+01],
             [1.500E-01 , 1.450E-01 , 1.399E-01 , 1.349E-01 , 1.312E-01 ,
             1.303E-01 , 1.288E-01 , 1.247E-01 , 1.185E-01 , 1.094E-01 ,
             9.962E-02 , 8.964E-02 , 7.814E-02 , 6.374E-02 , 5.025E-02 ,
             3.941E-02 , 3.069E-02 , 2.489E-02 , 1.966E-02 , 1.549E-02 ,
             1.331E-02 , 1.232E-02 , 1.232E-02 , 1.307E-02 , 1.400E-02 ,
             1.498E-02 , 1.598E-02 , 1.710E-02 , 1.850E-02 , 2.009E-02 ,
             2.220E-02 , 2.497E-02 , 2.824E-02 , 3.241E-02 , 3.717E-02 ,
             4.597E-02 , 6.639E-02 , 1.073E-01 , 1.862E-01 , 3.059E-01 ,
             6.375E-01 , 1.497E+00 , 3.239E+00 , 5.843E+00 , 1.013E+01 ,
             1.692E+01 , 2.467E+01 , 3.356E+01 , 4.148E+01 , 5.000E+01]])

#        CH4 /
ch4scale = 1.1
ch4 = np.array([[1.700E+00 , 1.700E+00 , 1.700E+00 , 1.700E+00 , 1.700E+00 ,
             1.700E+00 , 1.700E+00 , 1.699E+00 , 1.697E+00 , 1.693E+00 ,
             1.685E+00 , 1.675E+00 , 1.662E+00 , 1.645E+00 , 1.626E+00 ,
             1.605E+00 , 1.582E+00 , 1.553E+00 , 1.521E+00 , 1.480E+00 ,
             1.424E+00 , 1.355E+00 , 1.272E+00 , 1.191E+00 , 1.118E+00 ,
             1.055E+00 , 9.870E-01 , 9.136E-01 , 8.300E-01 , 7.460E-01 ,
             6.618E-01 , 5.638E-01 , 4.614E-01 , 3.631E-01 , 2.773E-01 ,
             2.100E-01 , 1.651E-01 , 1.500E-01 , 1.500E-01 , 1.500E-01 ,
             1.500E-01 , 1.500E-01 , 1.500E-01 , 1.400E-01 , 1.300E-01 ,
             1.200E-01 , 1.100E-01 , 9.500E-02 , 6.000E-02 , 3.000E-02],
             [1.700E+00 , 1.700E+00 , 1.700E+00 , 1.700E+00 , 1.697E+00 ,
             1.687E+00 , 1.672E+00 , 1.649E+00 , 1.629E+00 , 1.615E+00 ,
             1.579E+00 , 1.542E+00 , 1.508E+00 , 1.479E+00 , 1.451E+00 ,
             1.422E+00 , 1.390E+00 , 1.356E+00 , 1.323E+00 , 1.281E+00 ,
             1.224E+00 , 1.154E+00 , 1.066E+00 , 9.730E-01 , 8.800E-01 ,
             7.888E-01 , 7.046E-01 , 6.315E-01 , 5.592E-01 , 5.008E-01 ,
             4.453E-01 , 3.916E-01 , 3.389E-01 , 2.873E-01 , 2.384E-01 ,
             1.944E-01 , 1.574E-01 , 1.500E-01 , 1.500E-01 , 1.500E-01 ,
             1.500E-01 , 1.500E-01 , 1.500E-01 , 1.400E-01 , 1.300E-01 ,
             1.200E-01 , 1.100E-01 , 9.500E-02 , 6.000E-02 , 3.000E-02],
             [1.700E+00 , 1.700E+00 , 1.700E+00 , 1.700E+00 , 1.697E+00 ,
             1.687E+00 , 1.672E+00 , 1.649E+00 , 1.629E+00 , 1.615E+00 ,
             1.579E+00 , 1.542E+00 , 1.508E+00 , 1.479E+00 , 1.451E+00 ,
             1.422E+00 , 1.390E+00 , 1.356E+00 , 1.323E+00 , 1.281E+00 ,
             1.224E+00 , 1.154E+00 , 1.066E+00 , 9.730E-01 , 8.800E-01 ,
             7.931E-01 , 7.130E-01 , 6.438E-01 , 5.746E-01 , 5.050E-01 ,
             4.481E-01 , 3.931E-01 , 3.395E-01 , 2.876E-01 , 2.386E-01 ,
             1.944E-01 , 1.574E-01 , 1.500E-01 , 1.500E-01 , 1.500E-01 ,
             1.500E-01 , 1.500E-01 , 1.500E-01 , 1.400E-01 , 1.300E-01 ,
             1.200E-01 , 1.100E-01 , 9.500E-02 , 6.000E-02 , 3.000E-02],
             [1.700E+00 , 1.700E+00 , 1.700E+00 , 1.700E+00 , 1.697E+00 ,
             1.687E+00 , 1.672E+00 , 1.649E+00 , 1.629E+00 , 1.615E+00 ,
             1.579E+00 , 1.542E+00 , 1.506E+00 , 1.471E+00 , 1.434E+00 ,
             1.389E+00 , 1.342E+00 , 1.290E+00 , 1.230E+00 , 1.157E+00 ,
             1.072E+00 , 9.903E-01 , 9.170E-01 , 8.574E-01 , 8.013E-01 ,
             7.477E-01 , 6.956E-01 , 6.442E-01 , 5.888E-01 , 5.240E-01 ,
             4.506E-01 , 3.708E-01 , 2.992E-01 , 2.445E-01 , 2.000E-01 ,
             1.660E-01 , 1.500E-01 , 1.500E-01 , 1.500E-01 , 1.500E-01 ,
             1.500E-01 , 1.500E-01 , 1.500E-01 , 1.400E-01 , 1.300E-01 ,
             1.200E-01 , 1.100E-01 , 9.500E-02 , 6.000E-02 , 3.000E-02],
             [1.700E+00 , 1.700E+00 , 1.700E+00 , 1.700E+00 , 1.697E+00 ,
             1.687E+00 , 1.672E+00 , 1.649E+00 , 1.629E+00 , 1.615E+00 ,
             1.579E+00 , 1.542E+00 , 1.506E+00 , 1.471E+00 , 1.434E+00 ,
             1.389E+00 , 1.342E+00 , 1.290E+00 , 1.230E+00 , 1.161E+00 ,
             1.084E+00 , 1.014E+00 , 9.561E-01 , 9.009E-01 , 8.479E-01 ,
             7.961E-01 , 7.449E-01 , 6.941E-01 , 6.434E-01 , 5.883E-01 ,
             5.238E-01 , 4.505E-01 , 3.708E-01 , 3.004E-01 , 2.453E-01 ,
             1.980E-01 , 1.590E-01 , 1.500E-01 , 1.500E-01 , 1.500E-01 ,
             1.500E-01 , 1.500E-01 , 1.500E-01 , 1.400E-01 , 1.300E-01 ,
             1.200E-01 , 1.100E-01 , 9.500E-02 , 6.000E-02 , 3.000E-02],
             [1.700E+00 , 1.700E+00 , 1.700E+00 , 1.700E+00 , 1.700E+00 ,
             1.700E+00 , 1.700E+00 , 1.699E+00 , 1.697E+00 , 1.693E+00 ,
             1.685E+00 , 1.675E+00 , 1.662E+00 , 1.645E+00 , 1.626E+00 ,
             1.605E+00 , 1.582E+00 , 1.553E+00 , 1.521E+00 , 1.480E+00 ,
             1.424E+00 , 1.355E+00 , 1.272E+00 , 1.191E+00 , 1.118E+00 ,
             1.055E+00 , 9.870E-01 , 9.136E-01 , 8.300E-01 , 7.460E-01 ,
             6.618E-01 , 5.638E-01 , 4.614E-01 , 3.631E-01 , 2.773E-01 ,
             2.100E-01 , 1.650E-01 , 1.500E-01 , 1.500E-01 , 1.500E-01 ,
             1.500E-01 , 1.500E-01 , 1.500E-01 , 1.400E-01 , 1.300E-01 ,
             1.200E-01 , 1.100E-01 , 9.500E-02 , 6.000E-02 , 3.000E-02]]) * ch4scale

# ----------------------------------------------------------------------------
# MADE UP (NON-US76) profiles for other variables
# ----------------------------------------------------------------------------

# CLOUD LIQUID WATER
clw = np.array([[0., 0., .0001, .0001, .0001, 0., 0., 0., 0., 0.,
                 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
                 .0001, .0001, 0., 0., 0., 0., 0., 0., 0., 0.,
                 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
                 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
                [0., 0., .0, .0002, .0001, 0., 0., 0., 0., 0.,
                 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
                 .0001, .0001, 0., 0., 0., 0., 0., 0., 0., 0.,
                 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
                 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
                [0., 0., .0001, .0001, .0001, 0., 0., 0., 0., 0.,
                 0., 0., 0.0001, 0., 0., 0., 0., 0., 0., 0.,
                 .0001, .0001, 0., 0., 0., 0., 0., 0., 0., 0.,
                 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
                 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
                [0., 0., .0, .0001, .0001, 0., 0., 0., 0., 0.,
                 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
                 .0001, .0001, 0., 0., 0., 0., 0., 0., 0., 0.,
                 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
                 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
                [0., 0., .0002, .0001, .0, 0., 0., 0., 0., 0.,
                 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
                 .0001, .0001, 0., 0., 0., 0., 0., 0., 0., 0.,
                 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
                 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
                [0., 0., .0, .0002, .0001, 0., 0., 0., 0., 0.,
                 0., 0., 0.0001, 0., 0., 0., 0., 0., 0., 0.,
                 .0001, .0001, 0., 0., 0., 0., 0., 0., 0., 0.,
                 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
                 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.]])

# CLOUD, CFRAC and ICEDE

cld101 = np.array([
[0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.320, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.300, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.300, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.300, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.250, 0.000, 0.000, 0.020,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.250, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.250, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.200, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.200, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.200, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.130, 0.200, 0.000, 0.000,
 0.000, 0.000, 0.160, 0.300, 0.000, 0.000,
 0.000, 0.000, 0.160, 0.200, 0.000, 0.000,
 0.000, 0.000, 0.200, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.200, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.200, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.130, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.100, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.100, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.270, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.310, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.280, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.260, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.100, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.100, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.100, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.200, 0.000, 0.000, 0.200, 0.000, 0.000,
 0.200, 0.000, 0.000, 0.190, 0.000, 0.000,
 0.200, 0.000, 0.000, 0.180, 0.000, 0.000,
 0.210, 0.000, 0.200, 0.170, 0.000, 0.000,
 0.210, 0.000, 0.200, 0.150, 0.000, 0.000,
 0.210, 0.000, 0.150, 0.180, 0.000, 0.000,
 0.220, 0.000, 0.150, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.140, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.100, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.400, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000],

[0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.320, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.300, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.300, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.300, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.250, 0.020,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.250, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.250, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.200, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.200, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.200, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.130, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.160, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.160, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.200, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.200, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.200, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.130, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.100, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.100, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.270, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.310, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.280, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.260, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.100, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.100, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.100, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.200, 0.000, 0.000, 0.200, 0.000,
 0.000, 0.200, 0.000, 0.000, 0.190, 0.000,
 0.000, 0.200, 0.000, 0.000, 0.180, 0.000,
 0.000, 0.210, 0.000, 0.000, 0.170, 0.000,
 0.000, 0.210, 0.000, 0.000, 0.150, 0.000,
 0.000, 0.210, 0.000, 0.000, 0.180, 0.000,
 0.000, 0.220, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.140, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.100, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.400, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000],

[0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.320, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.300, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.300, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.300, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.250, 0.020,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.250, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.250, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.200, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.200, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.200, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.130, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.100, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.100, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.270, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.310, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.280, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.260, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.100, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.100, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.100, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.200, 0.000, 0.000, 0.200, 0.000,
 0.000, 0.200, 0.000, 0.000, 0.190, 0.000,
 0.000, 0.200, 0.000, 0.000, 0.180, 0.000,
 0.000, 0.210, 0.000, 0.000, 0.170, 0.000,
 0.000, 0.210, 0.000, 0.000, 0.150, 0.000,
 0.000, 0.210, 0.000, 0.000, 0.180, 0.000,
 0.000, 0.220, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.140, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.100, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.400, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000],

[0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.020,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.200, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.300, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.200, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.270, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.310, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.280, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.260, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.100, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.100, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.100, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.200, 0.000, 0.000, 0.200, 0.000, 0.000,
 0.200, 0.000, 0.000, 0.190, 0.000, 0.000,
 0.200, 0.000, 0.000, 0.180, 0.000, 0.000,
 0.210, 0.000, 0.200, 0.170, 0.000, 0.000,
 0.210, 0.000, 0.200, 0.150, 0.000, 0.000,
 0.210, 0.000, 0.150, 0.180, 0.000, 0.000,
 0.220, 0.000, 0.150, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.140, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.100, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.400, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000],

[0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.020,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.200, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.300, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.200, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.270, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.310, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.280, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.260, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.100, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.100, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.100, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.200, 0.000, 0.000, 0.200, 0.000, 0.000,
 0.200, 0.000, 0.000, 0.190, 0.000, 0.000,
 0.200, 0.000, 0.000, 0.180, 0.000, 0.000,
 0.210, 0.000, 0.200, 0.170, 0.000, 0.000,
 0.210, 0.000, 0.200, 0.150, 0.000, 0.000,
 0.210, 0.000, 0.150, 0.180, 0.000, 0.000,
 0.220, 0.000, 0.150, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.140, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.100, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.400, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000],

[0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.050,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.060,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.070,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.320, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.300, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.300, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.300, 0.000, 0.000, 0.080,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.250, 0.000, 0.000, 0.020,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.250, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.250, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.200, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.200, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.200, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.130, 0.200, 0.000, 0.000,
 0.000, 0.000, 0.160, 0.300, 0.000, 0.000,
 0.000, 0.000, 0.160, 0.200, 0.000, 0.000,
 0.000, 0.000, 0.200, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.200, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.200, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.130, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.100, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.100, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.270, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.310, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.280, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.260, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.100, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.100, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.100, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.200, 0.000, 0.000, 0.200, 0.000, 0.000,
 0.200, 0.000, 0.000, 0.190, 0.000, 0.000,
 0.200, 0.000, 0.000, 0.180, 0.000, 0.000,
 0.210, 0.000, 0.200, 0.170, 0.000, 0.000,
 0.210, 0.000, 0.200, 0.150, 0.000, 0.000,
 0.210, 0.000, 0.150, 0.180, 0.000, 0.000,
 0.220, 0.000, 0.150, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.140, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.100, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.400, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
 0.000, 0.000, 0.000, 0.000, 0.000, 0.000]])

cfrac101 = np.array([
 [0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.047,
  0.048,
  0.053,
  0.055,
  0.047,
  0.045,
  0.054,
  0.046,
  0.098,
  0.108,
  0.103,
  0.193,
  0.185,
  0.238,
  0.227,
  0.250,
  0.238,
  0.047,
  0.052,
  0.047,
  0.000,
  0.145,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.207,
  0.208,
  0.373,
  0.332,
  0.320,
  0.359,
  0.370,
  0.432,
  0.367,
  0.469,
  0.531,
  0.602,
  0.596,
  0.649,
  0.000,
  0.000,
  0.684,
  0.639,
  0.683,
  0.828,
  0.000,
  0.163,
  0.131,
  0.097,
  0.095,
  0.097,
  0.203,
  0.215,
  0.223,
  0.227,
  0.277,
  0.195,
  0.112,
  0.000,
  0.000,
  0.000,
  0.000,
  0.483,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000],

 [0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.050,
  0.054,
  0.048,
  0.047,
  0.051,
  0.050,
  0.054,
  0.049,
  0.092,
  0.104,
  0.099,
  0.182,
  0.187,
  0.237,
  0.225,
  0.230,
  0.248,
  0.045,
  0.047,
  0.050,
  0.000,
  0.140,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.180,
  0.195,
  0.325,
  0.366,
  0.353,
  0.332,
  0.418,
  0.418,
  0.377,
  0.488,
  0.533,
  0.650,
  0.561,
  0.545,
  0.000,
  0.000,
  1.000,
  1.000,
  1.000,
  1.000,
  0.000,
  0.142,
  0.133,
  0.106,
  0.093,
  0.108,
  0.200,
  0.205,
  0.230,
  0.259,
  0.320,
  0.198,
  0.118,
  0.000,
  0.000,
  0.000,
  0.000,
  0.548,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000],

 [0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.046,
  0.048,
  0.053,
  0.047,
  0.055,
  0.055,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.267,
  0.051,
  0.051,
  0.052,
  0.000,
  0.164,
  0.000,
  0.000,
  0.000,
  0.211,
  0.184,
  0.366,
  0.373,
  0.360,
  0.321,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.653,
  0.630,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.634,
  0.597,
  0.730,
  0.749,
  0.000,
  0.155,
  0.129,
  0.102,
  0.100,
  0.095,
  0.210,
  0.210,
  0.223,
  0.266,
  0.299,
  0.193,
  0.106,
  0.000,
  0.000,
  0.000,
  0.000,
  0.519,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000],

 [0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.048,
  0.055,
  0.052,
  0.050,
  0.054,
  0.047,
  0.046,
  0.047,
  0.108,
  0.097,
  0.101,
  0.189,
  0.190,
  0.226,
  0.256,
  0.218,
  0.245,
  0.054,
  0.055,
  0.055,
  0.000,
  0.157,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.372,
  0.391,
  0.379,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  1.000,
  1.000,
  1.000,
  1.000,
  0.000,
  0.153,
  0.134,
  0.108,
  0.101,
  0.108,
  0.182,
  0.202,
  1.000,
  1.000,
  1.000,
  0.190,
  0.103,
  0.000,
  0.000,
  0.000,
  0.000,
  0.503,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000],

 [0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.048,
  0.055,
  0.052,
  0.050,
  0.054,
  0.047,
  0.046,
  0.047,
  0.108,
  0.097,
  0.101,
  0.189,
  0.190,
  0.226,
  0.256,
  0.218,
  0.245,
  0.054,
  0.055,
  0.055,
  0.000,
  0.157,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.372,
  0.391,
  0.379,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  1.000,
  1.000,
  1.000,
  1.000,
  0.000,
  0.153,
  0.134,
  0.108,
  0.101,
  0.108,
  0.182,
  0.202,
  1.000,
  1.000,
  1.000,
  0.190,
  0.103,
  0.000,
  0.000,
  0.000,
  0.000,
  0.503,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000],

 [0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.047,
  0.048,
  0.053,
  0.055,
  0.047,
  0.045,
  0.054,
  0.046,
  0.098,
  0.108,
  0.103,
  0.193,
  0.185,
  0.238,
  0.227,
  0.250,
  0.238,
  0.047,
  0.052,
  0.047,
  0.000,
  0.145,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000,
  0.207,
  0.208,
  0.373,
  0.332,
  0.320,
  0.359,
  0.370,
  0.432,
  0.367,
  0.469,
  0.531,
  0.602,
  0.596,
  0.649,
  0.000,
  0.000,
  0.684,
  0.639,
  0.683,
  0.828,
  0.000,
  0.163,
  0.131,
  0.097,
  0.095,
  0.097,
  0.203,
  0.215,
  0.223,
  0.227,
  0.277,
  0.195,
  0.112,
  0.000,
  0.000,
  0.000,
  0.000,
  0.483,
  0.000,
  0.000,
  0.000,
  0.000,
  0.000]])

icede101 = np.array([  # If icede is negative the icede.txt file is not written out
  [-1.] * 100,

  [0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
  45.00,  50.00,  45.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,  50.00,  50.00,  55.00,  55.00,  55.00,
  60.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00],

  [0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,  50.00,
  55.00,  55.00,  50.00,   0.00,   0.00,   0.00,  50.00,  45.00,  45.00,  50.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00],

  [0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,  50.00,
  55.00,  55.00,  50.00,   0.00,   0.00,   0.00,  50.00,  45.00,  45.00,  50.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,
   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00],

  [-1.] * 100,
  [-1.] * 100])

# AEROSOL - currently profile 2 is a perturbed version of profile 1 and (3,4) and (5,6) are copies of (1,2).

aer101 = np.array([
   [0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0236,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0160,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0284,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9795,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9735,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0263,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0005,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9848,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0077,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0189,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0103,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0006,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0278,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9770,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9823,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0115,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9955,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0208,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0237,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0095,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0061,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9833,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9892,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9986,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9850,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9906,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9802,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0105,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9765,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0043,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9977,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9947,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0212,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0171,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0012,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9741,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0159,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9800,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0217,    0.0000,    0.0000,    0.0000,
    0.0131,   44.6446,   29.6945,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    1.6393,    0.0000,    0.0000,    0.0000,
    0.0292,  102.1250,   68.3850,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0303,  105.3668,   70.8860,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0312,  111.3181,   73.2864,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0333,  114.8135,   77.2800,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0337,  118.8004,   79.7853,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0353,  125.0307,   82.4940,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0368,  128.2791,   86.5725,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0379,  133.5048,   89.6555,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0400,  139.8668,   92.6937,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0413,  143.9810,   96.2061,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0433,  151.7655,  100.8641,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0444,  155.9996,  104.4377,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0456,  162.2947,  109.1611,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0482,  168.3892,  111.4787,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0491,  174.9958,  115.9539,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0510,  179.7233,  119.7951,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0528,  187.7244,  124.4031,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0546,  194.2988,  128.7888,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0576,  200.1216,  133.5540,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0158,   58.1784,   38.6703,    0.0000,    0.0000,   10.0623,    1.1562,    0.0050,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  118.8432,    0.0000,    0.0000,    0.0000,   15.8905,    1.7941,    0.0081,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  134.5163,    0.0000,    0.0000,    0.0000,   18.1668,    2.0721,    0.0100,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  155.2298,    0.0000,    0.0000,    0.0000,   20.7589,    2.3666,    0.0110,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  175.8670,    0.0000,    0.0000,    0.0000,   23.8183,    2.6826,    0.0130,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  203.6146,    0.0000,    0.0000,    0.0000,   27.1821,    3.1099,    0.0139,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  232.4947,    0.0000,    0.0000,    0.0000,   31.1192,    3.5072,    0.0161,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  263.0242,    0.0000,    0.0000,    0.0000,   35.7547,    4.0360,    0.0191,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  302.9584,    0.0000,    0.0000,    0.0000,   40.7281,    4.5768,    0.0211,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  339.8973,    0.0000,    0.0000,    0.0000,   46.6289,    5.2024,    0.0242,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  390.7113,    0.0000,    0.0000,    0.0000,   52.1237,    5.9880,    0.0278,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  439.0908,    0.0000,    0.0000,    0.0000,   59.8977,    6.7836,    0.0309,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  504.5881,    0.0000,    0.0000,    0.0000,   67.7189,    7.6704,    0.0357,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  569.2371,    0.0000,    0.0000,    0.0000,   77.0550,    8.6277,    0.0400,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  647.8057,    0.0000,    0.0000,    0.0000,   87.3869,    9.8368,    0.0456,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  734.7365,    0.0000,    0.0000,    0.0000,   98.7160,   11.1453,    0.0519,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  831.2908,    0.0000,    0.0000,    0.0000,  110.5260,   12.5619,    0.0593,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  936.4049,    0.0000,    0.0000,    0.0000,  126.5654,   14.0566,    0.0664,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1041.5405,    0.0000,    0.0000,    0.0000,  141.3612,   16.1121,    0.0754,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1192.9808,    0.0000,    0.0000,    0.0000,  157.8303,   18.0534,    0.0838,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1320.5907,    0.0000,    0.0000,    0.0000,  178.5866,   20.5080,    0.0959,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1503.4761,    0.0000,    0.0000,    0.0000,  202.0171,   22.8890,    0.1066,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1695.2749,    0.0000,    0.0000,    0.0000,  225.2138,   25.6943,    0.1190,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1873.9746,    0.0000,    0.0000,    0.0000,  252.8205,   28.6108,    0.1338,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000],

   [0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0257,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0136,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9972,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9820,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0024,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0057,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0089,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0208,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0133,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0116,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9744,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9998,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0003,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0019,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9803,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9986,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9840,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9846,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9980,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0147,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0167,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0066,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9767,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9896,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0110,    0.0000,    0.0000,    0.0000,
    0.0129,   44.6394,   29.4857,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    1.6361,    0.0000,    0.0000,    0.0000,
    0.0290,  101.8152,   68.3393,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0299,  106.7260,   69.9420,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0310,  111.1473,   74.2722,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0333,  113.9930,   76.5984,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0343,  119.3148,   79.9363,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0347,  123.2719,   83.0277,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0372,  128.7202,   86.8822,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0382,  133.4821,   90.4137,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0402,  141.0266,   92.2216,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0406,  144.6859,   95.9261,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0434,  150.4671,  101.2449,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0443,  156.4106,  103.9298,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0456,  161.5135,  107.4302,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0477,  167.0026,  111.7450,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0487,  175.5847,  115.2743,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0509,  180.9471,  121.7688,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0532,  186.5411,  125.1518,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0551,  193.2199,  129.8202,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0567,  200.9118,  133.5994,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0160,   57.3629,   38.4511,    0.0000,    0.0000,   10.2301,    1.1367,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  117.7044,    0.0000,    0.0000,    0.0000,   15.8594,    1.8111,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  134.5049,    0.0000,    0.0000,    0.0000,   18.3548,    2.0683,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  154.8746,    0.0000,    0.0000,    0.0000,   20.9666,    2.3507,    0.0111,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  179.0549,    0.0000,    0.0000,    0.0000,   23.9432,    2.7226,    0.0130,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  202.3033,    0.0000,    0.0000,    0.0000,   27.1596,    3.0716,    0.0141,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  229.9546,    0.0000,    0.0000,    0.0000,   31.5187,    3.5300,    0.0160,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  263.5522,    0.0000,    0.0000,    0.0000,   35.3439,    4.0410,    0.0190,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  301.4021,    0.0000,    0.0000,    0.0000,   40.7507,    4.6010,    0.0210,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  340.5703,    0.0000,    0.0000,    0.0000,   46.1890,    5.2455,    0.0240,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  392.6966,    0.0000,    0.0000,    0.0000,   52.2023,    5.9265,    0.0277,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  446.1153,    0.0000,    0.0000,    0.0000,   59.6042,    6.7282,    0.0308,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  499.0316,    0.0000,    0.0000,    0.0000,   67.6849,    7.6382,    0.0358,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  572.4213,    0.0000,    0.0000,    0.0000,   76.6934,    8.7584,    0.0404,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  643.5583,    0.0000,    0.0000,    0.0000,   86.4869,    9.8914,    0.0464,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  729.4867,    0.0000,    0.0000,    0.0000,   98.2906,   11.1453,    0.0523,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  816.1944,    0.0000,    0.0000,    0.0000,  112.1098,   12.6363,    0.0588,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  936.0286,    0.0000,    0.0000,    0.0000,  125.3701,   14.1755,    0.0664,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1055.1101,    0.0000,    0.0000,    0.0000,  140.7251,   16.1406,    0.0746,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1172.2663,    0.0000,    0.0000,    0.0000,  158.5576,   18.0594,    0.0833,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1336.1163,    0.0000,    0.0000,    0.0000,  180.3860,   20.4641,    0.0945,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1486.6578,    0.0000,    0.0000,    0.0000,  203.6619,   22.7603,    0.1050,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1696.1200,    0.0000,    0.0000,    0.0000,  226.4395,   25.4339,    0.1194,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1871.6901,    0.0000,    0.0000,    0.0000,  253.8597,   28.9626,    0.1339,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000],

   [0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0236,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0160,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0284,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9795,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9735,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0263,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0005,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9848,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0077,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0189,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0103,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0006,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0278,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9770,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9823,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0115,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9955,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0208,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0237,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0095,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0061,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9833,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9892,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9986,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9850,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9906,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9802,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0105,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9765,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0043,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9977,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9947,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0212,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0171,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0012,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9741,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0159,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9800,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0217,    0.0000,    0.0000,    0.0000,
    0.0131,   44.6446,   29.6945,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    1.6393,    0.0000,    0.0000,    0.0000,
    0.0292,  102.1250,   68.3850,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0303,  105.3668,   70.8860,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0312,  111.3181,   73.2864,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0333,  114.8135,   77.2800,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0337,  118.8004,   79.7853,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0353,  125.0307,   82.4940,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0368,  128.2791,   86.5725,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0379,  133.5048,   89.6555,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0400,  139.8668,   92.6937,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0413,  143.9810,   96.2061,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0433,  151.7655,  100.8641,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0444,  155.9996,  104.4377,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0456,  162.2947,  109.1611,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0482,  168.3892,  111.4787,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0491,  174.9958,  115.9539,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0510,  179.7233,  119.7951,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0528,  187.7244,  124.4031,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0546,  194.2988,  128.7888,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0576,  200.1216,  133.5540,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0158,   58.1784,   38.6703,    0.0000,    0.0000,   10.0623,    1.1562,    0.0050,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  118.8432,    0.0000,    0.0000,    0.0000,   15.8905,    1.7941,    0.0081,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  134.5163,    0.0000,    0.0000,    0.0000,   18.1668,    2.0721,    0.0100,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  155.2298,    0.0000,    0.0000,    0.0000,   20.7589,    2.3666,    0.0110,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  175.8670,    0.0000,    0.0000,    0.0000,   23.8183,    2.6826,    0.0130,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  203.6146,    0.0000,    0.0000,    0.0000,   27.1821,    3.1099,    0.0139,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  232.4947,    0.0000,    0.0000,    0.0000,   31.1192,    3.5072,    0.0161,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  263.0242,    0.0000,    0.0000,    0.0000,   35.7547,    4.0360,    0.0191,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  302.9584,    0.0000,    0.0000,    0.0000,   40.7281,    4.5768,    0.0211,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  339.8973,    0.0000,    0.0000,    0.0000,   46.6289,    5.2024,    0.0242,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  390.7113,    0.0000,    0.0000,    0.0000,   52.1237,    5.9880,    0.0278,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  439.0908,    0.0000,    0.0000,    0.0000,   59.8977,    6.7836,    0.0309,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  504.5881,    0.0000,    0.0000,    0.0000,   67.7189,    7.6704,    0.0357,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  569.2371,    0.0000,    0.0000,    0.0000,   77.0550,    8.6277,    0.0400,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  647.8057,    0.0000,    0.0000,    0.0000,   87.3869,    9.8368,    0.0456,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  734.7365,    0.0000,    0.0000,    0.0000,   98.7160,   11.1453,    0.0519,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  831.2908,    0.0000,    0.0000,    0.0000,  110.5260,   12.5619,    0.0593,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  936.4049,    0.0000,    0.0000,    0.0000,  126.5654,   14.0566,    0.0664,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1041.5405,    0.0000,    0.0000,    0.0000,  141.3612,   16.1121,    0.0754,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1192.9808,    0.0000,    0.0000,    0.0000,  157.8303,   18.0534,    0.0838,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1320.5907,    0.0000,    0.0000,    0.0000,  178.5866,   20.5080,    0.0959,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1503.4761,    0.0000,    0.0000,    0.0000,  202.0171,   22.8890,    0.1066,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1695.2749,    0.0000,    0.0000,    0.0000,  225.2138,   25.6943,    0.1190,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1873.9746,    0.0000,    0.0000,    0.0000,  252.8205,   28.6108,    0.1338,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000],

   [0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0257,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0136,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9972,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9820,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0024,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0057,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0089,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0208,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0133,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0116,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9744,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9998,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0003,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0019,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9803,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9986,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9840,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9846,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9980,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0147,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0167,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0066,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9767,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9896,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0110,    0.0000,    0.0000,    0.0000,
    0.0129,   44.6394,   29.4857,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    1.6361,    0.0000,    0.0000,    0.0000,
    0.0290,  101.8152,   68.3393,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0299,  106.7260,   69.9420,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0310,  111.1473,   74.2722,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0333,  113.9930,   76.5984,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0343,  119.3148,   79.9363,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0347,  123.2719,   83.0277,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0372,  128.7202,   86.8822,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0382,  133.4821,   90.4137,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0402,  141.0266,   92.2216,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0406,  144.6859,   95.9261,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0434,  150.4671,  101.2449,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0443,  156.4106,  103.9298,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0456,  161.5135,  107.4302,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0477,  167.0026,  111.7450,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0487,  175.5847,  115.2743,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0509,  180.9471,  121.7688,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0532,  186.5411,  125.1518,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0551,  193.2199,  129.8202,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0567,  200.9118,  133.5994,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0160,   57.3629,   38.4511,    0.0000,    0.0000,   10.2301,    1.1367,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  117.7044,    0.0000,    0.0000,    0.0000,   15.8594,    1.8111,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  134.5049,    0.0000,    0.0000,    0.0000,   18.3548,    2.0683,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  154.8746,    0.0000,    0.0000,    0.0000,   20.9666,    2.3507,    0.0111,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  179.0549,    0.0000,    0.0000,    0.0000,   23.9432,    2.7226,    0.0130,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  202.3033,    0.0000,    0.0000,    0.0000,   27.1596,    3.0716,    0.0141,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  229.9546,    0.0000,    0.0000,    0.0000,   31.5187,    3.5300,    0.0160,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  263.5522,    0.0000,    0.0000,    0.0000,   35.3439,    4.0410,    0.0190,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  301.4021,    0.0000,    0.0000,    0.0000,   40.7507,    4.6010,    0.0210,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  340.5703,    0.0000,    0.0000,    0.0000,   46.1890,    5.2455,    0.0240,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  392.6966,    0.0000,    0.0000,    0.0000,   52.2023,    5.9265,    0.0277,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  446.1153,    0.0000,    0.0000,    0.0000,   59.6042,    6.7282,    0.0308,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  499.0316,    0.0000,    0.0000,    0.0000,   67.6849,    7.6382,    0.0358,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  572.4213,    0.0000,    0.0000,    0.0000,   76.6934,    8.7584,    0.0404,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  643.5583,    0.0000,    0.0000,    0.0000,   86.4869,    9.8914,    0.0464,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  729.4867,    0.0000,    0.0000,    0.0000,   98.2906,   11.1453,    0.0523,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  816.1944,    0.0000,    0.0000,    0.0000,  112.1098,   12.6363,    0.0588,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  936.0286,    0.0000,    0.0000,    0.0000,  125.3701,   14.1755,    0.0664,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1055.1101,    0.0000,    0.0000,    0.0000,  140.7251,   16.1406,    0.0746,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1172.2663,    0.0000,    0.0000,    0.0000,  158.5576,   18.0594,    0.0833,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1336.1163,    0.0000,    0.0000,    0.0000,  180.3860,   20.4641,    0.0945,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1486.6578,    0.0000,    0.0000,    0.0000,  203.6619,   22.7603,    0.1050,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1696.1200,    0.0000,    0.0000,    0.0000,  226.4395,   25.4339,    0.1194,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1871.6901,    0.0000,    0.0000,    0.0000,  253.8597,   28.9626,    0.1339,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000],

   [0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0236,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0160,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0284,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9795,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9735,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0263,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0005,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9848,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0077,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0189,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0103,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0006,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0278,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9770,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9823,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0115,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9955,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0208,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0237,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0095,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0061,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9833,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9892,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9986,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9850,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9906,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9802,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0105,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9765,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0043,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9977,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9947,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0212,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0171,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0012,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9741,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0159,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9800,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0217,    0.0000,    0.0000,    0.0000,
    0.0131,   44.6446,   29.6945,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    1.6393,    0.0000,    0.0000,    0.0000,
    0.0292,  102.1250,   68.3850,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0303,  105.3668,   70.8860,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0312,  111.3181,   73.2864,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0333,  114.8135,   77.2800,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0337,  118.8004,   79.7853,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0353,  125.0307,   82.4940,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0368,  128.2791,   86.5725,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0379,  133.5048,   89.6555,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0400,  139.8668,   92.6937,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0413,  143.9810,   96.2061,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0433,  151.7655,  100.8641,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0444,  155.9996,  104.4377,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0456,  162.2947,  109.1611,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0482,  168.3892,  111.4787,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0491,  174.9958,  115.9539,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0510,  179.7233,  119.7951,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0528,  187.7244,  124.4031,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0546,  194.2988,  128.7888,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0576,  200.1216,  133.5540,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0158,   58.1784,   38.6703,    0.0000,    0.0000,   10.0623,    1.1562,    0.0050,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  118.8432,    0.0000,    0.0000,    0.0000,   15.8905,    1.7941,    0.0081,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  134.5163,    0.0000,    0.0000,    0.0000,   18.1668,    2.0721,    0.0100,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  155.2298,    0.0000,    0.0000,    0.0000,   20.7589,    2.3666,    0.0110,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  175.8670,    0.0000,    0.0000,    0.0000,   23.8183,    2.6826,    0.0130,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  203.6146,    0.0000,    0.0000,    0.0000,   27.1821,    3.1099,    0.0139,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  232.4947,    0.0000,    0.0000,    0.0000,   31.1192,    3.5072,    0.0161,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  263.0242,    0.0000,    0.0000,    0.0000,   35.7547,    4.0360,    0.0191,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  302.9584,    0.0000,    0.0000,    0.0000,   40.7281,    4.5768,    0.0211,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  339.8973,    0.0000,    0.0000,    0.0000,   46.6289,    5.2024,    0.0242,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  390.7113,    0.0000,    0.0000,    0.0000,   52.1237,    5.9880,    0.0278,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  439.0908,    0.0000,    0.0000,    0.0000,   59.8977,    6.7836,    0.0309,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  504.5881,    0.0000,    0.0000,    0.0000,   67.7189,    7.6704,    0.0357,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  569.2371,    0.0000,    0.0000,    0.0000,   77.0550,    8.6277,    0.0400,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  647.8057,    0.0000,    0.0000,    0.0000,   87.3869,    9.8368,    0.0456,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  734.7365,    0.0000,    0.0000,    0.0000,   98.7160,   11.1453,    0.0519,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  831.2908,    0.0000,    0.0000,    0.0000,  110.5260,   12.5619,    0.0593,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  936.4049,    0.0000,    0.0000,    0.0000,  126.5654,   14.0566,    0.0664,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1041.5405,    0.0000,    0.0000,    0.0000,  141.3612,   16.1121,    0.0754,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1192.9808,    0.0000,    0.0000,    0.0000,  157.8303,   18.0534,    0.0838,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1320.5907,    0.0000,    0.0000,    0.0000,  178.5866,   20.5080,    0.0959,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1503.4761,    0.0000,    0.0000,    0.0000,  202.0171,   22.8890,    0.1066,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1695.2749,    0.0000,    0.0000,    0.0000,  225.2138,   25.6943,    0.1190,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1873.9746,    0.0000,    0.0000,    0.0000,  252.8205,   28.6108,    0.1338,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000],

   [0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0257,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0136,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9972,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9820,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0024,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0057,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0089,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0208,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0133,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0116,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9744,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9998,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0003,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0019,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9803,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9986,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9840,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9846,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9980,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0147,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0167,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0066,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9767,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    2.9896,    0.0000,    0.0000,    0.0000,
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    3.0110,    0.0000,    0.0000,    0.0000,
    0.0129,   44.6394,   29.4857,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    1.6361,    0.0000,    0.0000,    0.0000,
    0.0290,  101.8152,   68.3393,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0299,  106.7260,   69.9420,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0310,  111.1473,   74.2722,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0333,  113.9930,   76.5984,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0343,  119.3148,   79.9363,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0347,  123.2719,   83.0277,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0372,  128.7202,   86.8822,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0382,  133.4821,   90.4137,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0402,  141.0266,   92.2216,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0406,  144.6859,   95.9261,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0434,  150.4671,  101.2449,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0443,  156.4106,  103.9298,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0456,  161.5135,  107.4302,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0477,  167.0026,  111.7450,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0487,  175.5847,  115.2743,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0509,  180.9471,  121.7688,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0532,  186.5411,  125.1518,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0551,  193.2199,  129.8202,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0567,  200.9118,  133.5994,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0160,   57.3629,   38.4511,    0.0000,    0.0000,   10.2301,    1.1367,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  117.7044,    0.0000,    0.0000,    0.0000,   15.8594,    1.8111,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  134.5049,    0.0000,    0.0000,    0.0000,   18.3548,    2.0683,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  154.8746,    0.0000,    0.0000,    0.0000,   20.9666,    2.3507,    0.0111,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  179.0549,    0.0000,    0.0000,    0.0000,   23.9432,    2.7226,    0.0130,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  202.3033,    0.0000,    0.0000,    0.0000,   27.1596,    3.0716,    0.0141,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  229.9546,    0.0000,    0.0000,    0.0000,   31.5187,    3.5300,    0.0160,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  263.5522,    0.0000,    0.0000,    0.0000,   35.3439,    4.0410,    0.0190,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  301.4021,    0.0000,    0.0000,    0.0000,   40.7507,    4.6010,    0.0210,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  340.5703,    0.0000,    0.0000,    0.0000,   46.1890,    5.2455,    0.0240,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  392.6966,    0.0000,    0.0000,    0.0000,   52.2023,    5.9265,    0.0277,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  446.1153,    0.0000,    0.0000,    0.0000,   59.6042,    6.7282,    0.0308,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  499.0316,    0.0000,    0.0000,    0.0000,   67.6849,    7.6382,    0.0358,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  572.4213,    0.0000,    0.0000,    0.0000,   76.6934,    8.7584,    0.0404,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  643.5583,    0.0000,    0.0000,    0.0000,   86.4869,    9.8914,    0.0464,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  729.4867,    0.0000,    0.0000,    0.0000,   98.2906,   11.1453,    0.0523,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  816.1944,    0.0000,    0.0000,    0.0000,  112.1098,   12.6363,    0.0588,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000,  936.0286,    0.0000,    0.0000,    0.0000,  125.3701,   14.1755,    0.0664,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1055.1101,    0.0000,    0.0000,    0.0000,  140.7251,   16.1406,    0.0746,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1172.2663,    0.0000,    0.0000,    0.0000,  158.5576,   18.0594,    0.0833,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1336.1163,    0.0000,    0.0000,    0.0000,  180.3860,   20.4641,    0.0945,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1486.6578,    0.0000,    0.0000,    0.0000,  203.6619,   22.7603,    0.1050,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1696.1200,    0.0000,    0.0000,    0.0000,  226.4395,   25.4339,    0.1194,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,
    0.0000, 1871.6901,    0.0000,    0.0000,    0.0000,  253.8597,   28.9626,    0.1339,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000]])

# ----------------------------------------------------------------------------
# Additional profile paramaters
# ----------------------------------------------------------------------------

# NPROF = number of profiles per set (i.e number of US76 profiles)
# NLEV  = number of levels in US76 profiles
NPROF, NLEV = p.shape

# 2m p varies above and below 1050hPa so that for the standard 54L and varying101L there
# are profiles with the surface both above and below the bottom level of the profile
# 2m T, q and o3 and skin T: some are the same as in bottom level, the others differ slightly

# ----------------------------------------------------------------------------------------------
# Profile number                     1          2           3          4          5          6
# ----------------------------------------------------------------------------------------------
satzen                     = [      45.,       50.,        45.,       55.,        0.,       60.]
satazi                     = [       0.,      185.,       170.,       25.,       90.,       45.]
sunzen                     = [      40.,       55.,        35.,       15.,       70.,        0.]
sunazi                     = [     179.,      179.,       179.,      120.,       90.,        0.]

lat                        = [      15.,       45.,        45.,       60.,       60.,      -45.]
elev                       = [       0.,        0.,         0.,       0.5,        0.,       0.2]

ctp                        = [     949.,      510.,       500.,      500.,      400.,      500.]
cfraction                  = [      0.6,       0.1,        0.5,       0.0,       0.2,       1.0]

be                         = [      0.2,       0.5,        0.7,       0.2,       0.5,       0.7]
cosbk                      = [       0.,        0.,        0.5,       0.5,       1.0,       1.0]

ish                        = [        1,         1,          2,         2,         3,         4]
idg                        = [        1,         2,          3,         4,         4,         4]

s2m = {}
s2m['p']                   = [     950.,     1013.,      1100.,     1050.,     1030.,     1100.]
s2m['t']                   = [    298.7,     295.2,      272.2,     287.2,     256.2,     288.2]
s2m['q']                   = [  24930.0,   18760.0,     4416.0,   12940.0,    1405.0,    7545.0]
s2m['o']                   = [  0.02869,   0.03317,    0.02778,   0.02112,   0.01802,    0.0166]
s2m['u']                   = [       5.,        0.,        -3.,        4.,       10.,       -6.]
s2m['v']                   = [       2.,       -7.,        -1.,        1.,        3.,        0.]
s2m['wfetc']               = [  100000.,    50000.,    200000.,   100000.,    50000.,   200000.]

skin = {}
skin['t']                  = [     302.,     294.8,      272.2,     295.2,     257.2,     286.2]
skin['salinity']           = [      35.,       33.,        37.,       35.,       33.,       37.]
skin['surftype']           = [        1,         1,          1,         0,         2,         0]
skin['surftype_allsea']    = [        1,         1,          1,         1,         1,         1]  # For sea-only profile sets
skin['watertype']          = [        0,         1,          1,         0,         1,         1]
skin['fastem']             = [[3., 5., 15., 0.1, 0.3]] * 6

datetime                   = [[1949,  1,  1,  0,  0,  0], # First date is out of range for testing purposes
                              [1966,  7,  1, 17,  0,  0], # Times are arbitrary (not currently used by code)
                              [1966,  1,  1, 12, 30, 10],
                              [1966,  7,  1, 23,  0,  0],
                              [1966,  1,  1,  0, 25,  0],
                              [1976,  7,  1,  9, 45,  0]]

# ----------------------------------------------------------------------------
# RTTOV coefficient pressure levels
# ----------------------------------------------------------------------------

p101 = np.array(
[   0.0050,    0.0161,    0.0384,    0.0769,    0.1370,    0.2244,    0.3454,    0.5064,
    0.7140,    0.9753,    1.2972,    1.6872,    2.1526,    2.7009,    3.3398,    4.0770,
    4.9204,    5.8776,    6.9567,    8.1655,    9.5119,   11.0038,   12.6492,   14.4559,
   16.4318,   18.5847,   20.9224,   23.4526,   26.1829,   29.1210,   32.2744,   35.6505,
   39.2566,   43.1001,   47.1882,   51.5278,   56.1260,   60.9895,   66.1253,   71.5398,
   77.2396,   83.2310,   89.5204,   96.1138,  103.0172,  110.2366,  117.7775,  125.6456,
  133.8462,  142.3848,  151.2664,  160.4959,  170.0784,  180.0183,  190.3203,  200.9887,
  212.0277,  223.4415,  235.2338,  247.4085,  259.9691,  272.9191,  286.2617,  300.0000,
  314.1369,  328.6753,  343.6176,  358.9665,  374.7241,  390.8926,  407.4738,  424.4698,
  441.8819,  459.7118,  477.9607,  496.6298,  515.7200,  535.2322,  555.1669,  575.5248,
  596.3062,  617.5112,  639.1398,  661.1920,  683.6673,  706.5654,  729.8857,  753.6275,
  777.7897,  802.3714,  827.3713,  852.7880,  878.6201,  904.8659,  931.5236,  958.5911,
  986.0666, 1013.9476, 1042.2319, 1070.9170, 1100.0000])

p54 = np.array(
[   0.0050,    0.0131,    0.0304,    0.0644,    0.1263,    0.2324,    0.4052,    0.6749,
    1.0801,    1.6691,    2.5011,    3.6462,    5.1864,    7.2150,    9.8368,   13.1672,
   17.3308,   22.4601,   28.6937,   36.1735,   45.0430,   55.4433,   67.5109,   81.3744,
   97.1505,  114.9415,  134.8318,  156.8846,  181.1394,  207.6092,  236.2784,  267.1012,
  300.0000,  334.8648,  371.5529,  409.8893,  449.6677,  490.6516,  532.5769,  575.1538,
  618.0706,  660.9965,  703.5863,  745.4841,  786.3278,  825.7546,  863.4047,  898.9275,
  931.9853,  962.2587,  989.4510, 1013.2923, 1033.5436, 1050.0000])

p51 = np.array(
[   0.0050,    0.0100,    0.1000,    0.2000,    0.5000,    0.8000,    1.2000,    1.6000,
    2.2000,    2.7000,    3.5000,    4.2000,    5.0000,    6.9500,   10.3700,   14.8100,
   20.4000,   27.2600,   35.5100,   45.2900,   56.7300,   69.9700,   85.1800,  102.0500,
  122.0400,  143.8400,  167.9500,  194.3600,  222.9400,  253.7100,  286.6000,  321.5000,
  358.2800,  396.8100,  436.9500,  478.5400,  521.4600,  565.5400,  610.6000,  656.4300,
  702.7300,  749.1200,  795.0900,  839.9500,  882.8000,  922.4600,  957.4400,  985.8800,
 1005.4300, 1025.0000, 1050.0000])

p44 = np.array(
[   0.0050,    0.1000,    0.2900,    0.6900,    1.4200,    2.6110,    4.4070,    6.9500,
   10.3700,   14.8100,   20.4000,   27.2600,   35.5100,   45.2900,   56.7300,   69.9700,
   85.1800,  102.0500,  122.0400,  143.8400,  167.9500,  194.3600,  222.9400,  253.7100,
  286.6000,  321.5000,  358.2800,  396.8100,  436.9500,  478.5400,  521.4600,  565.5400,
  610.6000,  656.4300,  702.7300,  749.1200,  795.0900,  839.9500,  882.8000,  922.4600,
  957.4400,  985.8800, 1005.4300, 1013.2500])

# ----------------------------------------------------------------------------
# RTTOV reference gas profiles 101L
# ----------------------------------------------------------------------------

# Some trace gas reference profiles changed with the new all-trace-gas v9 predictor files in February 2014.
# The "new" trace gas profile are the ones used in 101L v9 predictor all-trace-gas files.
# The commented out gases are those used in 101L v8 predictor files (this only affects CO2 in practice).

# New CH4 reference
ch4ref101 = np.array(
[1.568340E-01, 1.541630E-01, 1.578910E-01, 1.692510E-01, 1.965060E-01, 2.345350E-01, 2.708110E-01, 3.049250E-01,
 3.446060E-01, 3.965290E-01, 4.699220E-01, 5.523530E-01, 6.388540E-01, 7.362970E-01, 8.233940E-01, 8.947100E-01,
 9.569370E-01, 1.003380E+00, 1.046410E+00, 1.079670E+00, 1.111100E+00, 1.151380E+00, 1.195110E+00, 1.237410E+00,
 1.285360E+00, 1.332740E+00, 1.378340E+00, 1.418230E+00, 1.455340E+00, 1.479230E+00, 1.504880E+00, 1.532440E+00,
 1.561900E+00, 1.584910E+00, 1.607090E+00, 1.627420E+00, 1.646150E+00, 1.662800E+00, 1.673420E+00, 1.684620E+00,
 1.696390E+00, 1.708760E+00, 1.721700E+00, 1.746750E+00, 1.761310E+00, 1.776510E+00, 1.788440E+00, 1.797800E+00,
 1.806820E+00, 1.814730E+00, 1.822450E+00, 1.827690E+00, 1.832370E+00, 1.836370E+00, 1.839410E+00, 1.842210E+00,
 1.844430E+00, 1.846540E+00, 1.848410E+00, 1.850180E+00, 1.851880E+00, 1.853590E+00, 1.855150E+00, 1.856330E+00,
 1.857370E+00, 1.858160E+00, 1.858820E+00, 1.859440E+00, 1.860090E+00, 1.860660E+00, 1.861240E+00, 1.861800E+00,
 1.862410E+00, 1.863060E+00, 1.863700E+00, 1.864380E+00, 1.865260E+00, 1.866180E+00, 1.867010E+00, 1.867860E+00,
 1.868510E+00, 1.869130E+00, 1.869790E+00, 1.870470E+00, 1.871190E+00, 1.872020E+00, 1.873140E+00, 1.874380E+00,
 1.875780E+00, 1.877460E+00, 1.878740E+00, 1.879870E+00, 1.881100E+00, 1.882230E+00, 1.883000E+00, 1.883550E+00,
 1.884010E+00, 1.884320E+00, 1.884520E+00, 1.884600E+00, 1.884620E+00])

# Old CH4 reference
#ch4ref101 = np.array(
#[1.505280E-01, 1.480550E-01, 1.517880E-01, 1.627430E-01, 1.890330E-01, 2.256860E-01, 2.604690E-01, 2.931800E-01,
 #3.313100E-01, 3.813640E-01, 4.521410E-01, 5.316540E-01, 6.151010E-01, 7.089150E-01, 7.927580E-01, 8.614090E-01,
 #9.212740E-01, 9.660020E-01, 1.007440E+00, 1.039500E+00, 1.069780E+00, 1.108640E+00, 1.150840E+00, 1.191630E+00,
 #1.237930E+00, 1.283660E+00, 1.327680E+00, 1.366130E+00, 1.401900E+00, 1.424890E+00, 1.449570E+00, 1.476090E+00,
 #1.504430E+00, 1.526560E+00, 1.547880E+00, 1.567450E+00, 1.585460E+00, 1.601480E+00, 1.611700E+00, 1.622470E+00,
 #1.633790E+00, 1.645680E+00, 1.658120E+00, 1.682210E+00, 1.696250E+00, 1.710920E+00, 1.722430E+00, 1.731470E+00,
 #1.740160E+00, 1.747790E+00, 1.755210E+00, 1.760250E+00, 1.764760E+00, 1.768620E+00, 1.771540E+00, 1.774240E+00,
 #1.776360E+00, 1.778370E+00, 1.780150E+00, 1.781830E+00, 1.783440E+00, 1.785040E+00, 1.786500E+00, 1.787600E+00,
 #1.788570E+00, 1.789290E+00, 1.789910E+00, 1.790480E+00, 1.791080E+00, 1.791600E+00, 1.792150E+00, 1.792670E+00,
 #1.793230E+00, 1.793830E+00, 1.794430E+00, 1.795070E+00, 1.795890E+00, 1.796740E+00, 1.797520E+00, 1.798310E+00,
 #1.798930E+00, 1.799500E+00, 1.800120E+00, 1.800760E+00, 1.801430E+00, 1.802200E+00, 1.803250E+00, 1.804410E+00,
 #1.805730E+00, 1.807310E+00, 1.808520E+00, 1.809570E+00, 1.810730E+00, 1.811790E+00, 1.812510E+00, 1.813030E+00,
 #1.813460E+00, 1.813760E+00, 1.813940E+00, 1.814020E+00, 1.814040E+00])


# No change in CO reference
coref101 = np.array(
[8.447250E-01, 8.264420E-01, 7.912930E-01, 7.352120E-01, 6.580020E-01, 5.646420E-01, 4.423180E-01, 3.203560E-01,
 2.455140E-01, 1.973310E-01, 1.556390E-01, 1.163780E-01, 8.853060E-02, 7.106010E-02, 6.020340E-02, 5.133640E-02,
 4.390530E-02, 4.121090E-02, 3.961650E-02, 3.772260E-02, 3.507200E-02, 3.278300E-02, 3.065800E-02, 2.824910E-02,
 2.647490E-02, 2.499730E-02, 2.425810E-02, 2.376840E-02, 2.366680E-02, 2.362390E-02, 2.377130E-02, 2.398640E-02,
 2.447070E-02, 2.515050E-02, 2.638660E-02, 2.806340E-02, 3.028820E-02, 3.346500E-02, 3.736270E-02, 4.144510E-02,
 4.627920E-02, 5.160830E-02, 5.744700E-02, 6.445700E-02, 6.799680E-02, 7.253380E-02, 7.638010E-02, 7.976440E-02,
 8.339460E-02, 8.597180E-02, 8.887220E-02, 9.134090E-02, 9.401590E-02, 9.719320E-02, 1.008300E-01, 1.048430E-01,
 1.089410E-01, 1.134990E-01, 1.177400E-01, 1.219100E-01, 1.258460E-01, 1.290080E-01, 1.319110E-01, 1.340770E-01,
 1.361580E-01, 1.376490E-01, 1.389830E-01, 1.399070E-01, 1.406500E-01, 1.411770E-01, 1.415820E-01, 1.419190E-01,
 1.424450E-01, 1.429710E-01, 1.433510E-01, 1.437380E-01, 1.441670E-01, 1.445870E-01, 1.448040E-01, 1.450590E-01,
 1.451620E-01, 1.454250E-01, 1.462610E-01, 1.476070E-01, 1.489880E-01, 1.498720E-01, 1.504290E-01, 1.510610E-01,
 1.516930E-01, 1.529860E-01, 1.546270E-01, 1.569740E-01, 1.593760E-01, 1.612940E-01, 1.639430E-01, 1.668430E-01,
 1.679650E-01, 1.692540E-01, 1.708910E-01, 1.727530E-01, 1.748610E-01])


# New CO2 reference
co2ref101 = np.array(
[3.859550E+02, 3.859590E+02, 3.859700E+02, 3.859880E+02, 3.860160E+02, 3.860560E+02, 3.861090E+02, 3.861730E+02,
 3.862380E+02, 3.862400E+02, 3.862000E+02, 3.861510E+02, 3.862250E+02, 3.863960E+02, 3.865850E+02, 3.868080E+02,
 3.870390E+02, 3.872500E+02, 3.874510E+02, 3.876450E+02, 3.877830E+02, 3.878610E+02, 3.879100E+02, 3.879310E+02,
 3.879520E+02, 3.879720E+02, 3.880550E+02, 3.881580E+02, 3.883570E+02, 3.885750E+02, 3.888650E+02, 3.891760E+02,
 3.895330E+02, 3.899140E+02, 3.901740E+02, 3.903850E+02, 3.906500E+02, 3.910220E+02, 3.914150E+02, 3.920300E+02,
 3.926980E+02, 3.932490E+02, 3.936530E+02, 3.940680E+02, 3.942120E+02, 3.943600E+02, 3.942610E+02, 3.939690E+02,
 3.939720E+02, 3.943670E+02, 3.947740E+02, 3.950810E+02, 3.953670E+02, 3.956070E+02, 3.957870E+02, 3.959460E+02,
 3.959870E+02, 3.960200E+02, 3.958900E+02, 3.959390E+02, 3.962340E+02, 3.965020E+02, 3.967790E+02, 3.969810E+02,
 3.971710E+02, 3.973370E+02, 3.974940E+02, 3.976390E+02, 3.977780E+02, 3.979010E+02, 3.980330E+02, 3.981640E+02,
 3.983190E+02, 3.984840E+02, 3.986520E+02, 3.988220E+02, 3.989700E+02, 3.991080E+02, 3.992190E+02, 3.993210E+02,
 3.993950E+02, 3.994660E+02, 3.995280E+02, 3.995870E+02, 3.996440E+02, 3.997040E+02, 3.997440E+02, 3.997890E+02,
 3.998810E+02, 4.000290E+02, 4.001980E+02, 4.003690E+02, 4.005620E+02, 4.007520E+02, 4.009400E+02, 4.010970E+02,
 4.012240E+02, 4.013150E+02, 4.013740E+02, 4.013980E+02, 4.014020E+02])

# Old CO2 reference
#co2ref101 = np.array(
#[3.743610E+02, 3.743620E+02, 3.743650E+02, 3.743690E+02, 3.743750E+02, 3.743850E+02, 3.743970E+02, 3.744130E+02,
 #3.744430E+02, 3.744910E+02, 3.745510E+02, 3.746090E+02, 3.746490E+02, 3.746810E+02, 3.747170E+02, 3.747620E+02,
 #3.748240E+02, 3.748990E+02, 3.749780E+02, 3.750540E+02, 3.751160E+02, 3.751650E+02, 3.752080E+02, 3.752400E+02,
 #3.752750E+02, 3.753130E+02, 3.753700E+02, 3.754350E+02, 3.755320E+02, 3.756390E+02, 3.757780E+02, 3.759260E+02,
 #3.761000E+02, 3.762860E+02, 3.764820E+02, 3.766890E+02, 3.769300E+02, 3.772320E+02, 3.775510E+02, 3.779150E+02,
 #3.783020E+02, 3.786600E+02, 3.789800E+02, 3.793130E+02, 3.795350E+02, 3.797680E+02, 3.799770E+02, 3.801690E+02,
 #3.803620E+02, 3.805440E+02, 3.807340E+02, 3.808930E+02, 3.810480E+02, 3.812020E+02, 3.813590E+02, 3.815150E+02,
 #3.816730E+02, 3.818330E+02, 3.819890E+02, 3.821410E+02, 3.822860E+02, 3.824130E+02, 3.825350E+02, 3.826190E+02,
 #3.826940E+02, 3.827520E+02, 3.828030E+02, 3.828430E+02, 3.828800E+02, 3.829120E+02, 3.829460E+02, 3.829800E+02,
 #3.830220E+02, 3.830660E+02, 3.831080E+02, 3.831490E+02, 3.831800E+02, 3.832080E+02, 3.832280E+02, 3.832460E+02,
 #3.832580E+02, 3.832720E+02, 3.832840E+02, 3.832990E+02, 3.833180E+02, 3.833390E+02, 3.833550E+02, 3.833750E+02,
 #3.834140E+02, 3.834760E+02, 3.835460E+02, 3.836170E+02, 3.836960E+02, 3.837780E+02, 3.838530E+02, 3.839160E+02,
 #3.839670E+02, 3.840030E+02, 3.840270E+02, 3.840360E+02, 3.840380E+02])


# New N2O reference
n2oref101 = np.array(
[8.283970E-03, 6.146450E-03, 4.600580E-03, 3.617280E-03, 2.978870E-03, 2.134980E-03, 1.932400E-03, 2.248220E-03,
 4.169530E-03, 6.836570E-03, 9.896040E-03, 1.277160E-02, 1.538140E-02, 2.038530E-02, 2.561540E-02, 3.207970E-02,
 3.888020E-02, 4.670070E-02, 5.464390E-02, 6.488300E-02, 7.463770E-02, 8.639650E-02, 9.886690E-02, 1.108150E-01,
 1.243580E-01, 1.376660E-01, 1.505160E-01, 1.617890E-01, 1.722850E-01, 1.823530E-01, 1.916250E-01, 1.990450E-01,
 2.068980E-01, 2.144440E-01, 2.221340E-01, 2.295220E-01, 2.365360E-01, 2.434010E-01, 2.505910E-01, 2.579310E-01,
 2.646060E-01, 2.721330E-01, 2.793860E-01, 2.860540E-01, 2.922630E-01, 2.979270E-01, 3.028370E-01, 3.070890E-01,
 3.107830E-01, 3.136960E-01, 3.163190E-01, 3.185760E-01, 3.203900E-01, 3.219490E-01, 3.231670E-01, 3.241990E-01,
 3.250430E-01, 3.256510E-01, 3.261210E-01, 3.264640E-01, 3.266440E-01, 3.267680E-01, 3.268600E-01, 3.269260E-01,
 3.269680E-01, 3.269900E-01, 3.270000E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01,
 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01,
 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01,
 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01,
 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01, 3.270030E-01])

# Old N2O reference
#n2oref101 = np.array(
#[8.143570E-03, 6.042910E-03, 4.523420E-03, 3.556200E-03, 2.928400E-03, 2.097760E-03, 1.899340E-03, 2.208960E-03,
 #4.096410E-03, 6.718630E-03, 9.726530E-03, 1.255440E-02, 1.512100E-02, 2.003750E-02, 2.517740E-02, 3.153040E-02,
 #3.821580E-02, 4.591190E-02, 5.372470E-02, 6.379570E-02, 7.338980E-02, 8.496010E-02, 9.723330E-02, 1.089920E-01,
 #1.223230E-01, 1.354200E-01, 1.480660E-01, 1.591560E-01, 1.694790E-01, 1.793820E-01, 1.885010E-01, 1.957960E-01,
 #2.035240E-01, 2.109500E-01, 2.185210E-01, 2.257940E-01, 2.326980E-01, 2.394560E-01, 2.465370E-01, 2.537660E-01,
 #2.603370E-01, 2.677490E-01, 2.748900E-01, 2.814540E-01, 2.875650E-01, 2.931410E-01, 2.979760E-01, 3.021620E-01,
 #3.057990E-01, 3.086680E-01, 3.112500E-01, 3.134730E-01, 3.152590E-01, 3.167940E-01, 3.179950E-01, 3.190120E-01,
 #3.198420E-01, 3.204430E-01, 3.209060E-01, 3.212440E-01, 3.214210E-01, 3.215440E-01, 3.216340E-01, 3.216990E-01,
 #3.217400E-01, 3.217610E-01, 3.217720E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01,
 #3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01,
 #3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01,
 #3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01,
 #3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01, 3.217740E-01])


# No change in O3 reference
o3ref101 = np.array(
[2.961660E-01, 3.260630E-01, 4.005310E-01, 5.700460E-01, 8.017010E-01, 1.050990E+00, 1.334230E+00, 1.661750E+00,
 2.055670E+00, 2.571570E+00, 3.173120E+00, 3.781340E+00, 4.432110E+00, 5.085490E+00, 5.705740E+00, 6.268520E+00,
 6.684410E+00, 6.948240E+00, 7.098700E+00, 7.144780E+00, 7.095520E+00, 6.942250E+00, 6.681230E+00, 6.323640E+00,
 5.892930E+00, 5.418060E+00, 4.963980E+00, 4.546650E+00, 4.171290E+00, 3.820790E+00, 3.477880E+00, 3.153750E+00,
 2.869420E+00, 2.603360E+00, 2.345530E+00, 2.099750E+00, 1.874750E+00, 1.674110E+00, 1.488140E+00, 1.305870E+00,
 1.130740E+00, 9.723940E-01, 8.396630E-01, 7.443660E-01, 6.726360E-01, 6.248520E-01, 5.917570E-01, 5.461750E-01,
 4.944430E-01, 4.513150E-01, 4.110930E-01, 3.729810E-01, 3.306930E-01, 2.886800E-01, 2.493390E-01, 2.156170E-01,
 1.866140E-01, 1.641360E-01, 1.464090E-01, 1.304190E-01, 1.168380E-01, 1.043180E-01, 9.421150E-02, 8.629350E-02,
 8.011040E-02, 7.474630E-02, 6.971460E-02, 6.570650E-02, 6.249080E-02, 5.994800E-02, 5.765790E-02, 5.594840E-02,
 5.426770E-02, 5.305050E-02, 5.229980E-02, 5.147090E-02, 5.078830E-02, 5.013580E-02, 4.986130E-02, 4.954940E-02,
 4.935300E-02, 4.914240E-02, 4.849530E-02, 4.757890E-02, 4.731710E-02, 4.711830E-02, 4.677570E-02, 4.586570E-02,
 4.496860E-02, 4.399520E-02, 4.298690E-02, 4.189290E-02, 4.089820E-02, 3.966980E-02, 3.764950E-02, 3.495680E-02,
 3.074920E-02, 2.777480E-02, 2.736080E-02, 2.736080E-02, 2.736080E-02])

# ----------------------------------------------------------------------------
# RTTOV reference gas profiles 54L
# ----------------------------------------------------------------------------

co2ref54 = np.array(
[3.743610E+02, 3.743620E+02, 3.743640E+02, 3.743680E+02, 3.743740E+02, 3.743860E+02, 3.744040E+02, 3.744380E+02,
 3.745120E+02, 3.746070E+02, 3.746700E+02, 3.747370E+02, 3.748460E+02, 3.749950E+02, 3.751270E+02, 3.752180E+02,
 3.752910E+02, 3.754100E+02, 3.756240E+02, 3.759520E+02, 3.763810E+02, 3.768950E+02, 3.776470E+02, 3.785520E+02,
 3.793470E+02, 3.799000E+02, 3.803840E+02, 3.808320E+02, 3.812200E+02, 3.816110E+02, 3.820020E+02, 3.823570E+02,
 3.826190E+02, 3.827730E+02, 3.828730E+02, 3.829510E+02, 3.830410E+02, 3.831360E+02, 3.832040E+02, 3.832460E+02,
 3.832720E+02, 3.832990E+02, 3.833360E+02, 3.833680E+02, 3.834360E+02, 3.835420E+02, 3.836500E+02, 3.837600E+02,
 3.838540E+02, 3.839230E+02, 3.839710E+02, 3.840020E+02, 3.840200E+02, 3.840290E+02])

o3ref54 = np.array(
[2.961660E-01, 3.207910E-01, 3.805170E-01, 5.267450E-01, 7.690790E-01, 1.074000E+00, 1.470910E+00, 1.991100E+00,
 2.786830E+00, 3.756380E+00, 4.864190E+00, 5.953410E+00, 6.762550E+00, 7.109190E+00, 7.060190E+00, 6.573730E+00,
 5.687480E+00, 4.704720E+00, 3.869510E+00, 3.110780E+00, 2.477910E+00, 1.906960E+00, 1.440110E+00, 1.020210E+00,
 7.332710E-01, 6.039500E-01, 4.893260E-01, 3.876250E-01, 2.842900E-01, 1.980380E-01, 1.450050E-01, 1.098670E-01,
 8.629350E-02, 7.263480E-02, 6.312690E-02, 5.741070E-02, 5.373030E-02, 5.173290E-02, 5.022320E-02, 4.955500E-02,
 4.912540E-02, 4.758690E-02, 4.714380E-02, 4.617460E-02, 4.462710E-02, 4.305120E-02, 4.148050E-02, 3.994460E-02,
 3.760300E-02, 3.438820E-02, 3.038380E-02, 2.784400E-02, 2.748680E-02, 2.736080E-02])

# ----------------------------------------------------------------------------
# RTTOV reference gas profiles 51L
# ----------------------------------------------------------------------------

co2ref51 = np.array(
[3.743610E+02, 3.743620E+02, 3.743720E+02, 3.743830E+02, 3.744120E+02, 3.744600E+02, 3.745350E+02, 3.745970E+02,
 3.746520E+02, 3.746810E+02, 3.747280E+02, 3.747720E+02, 3.748310E+02, 3.749780E+02, 3.751450E+02, 3.752470E+02,
 3.753580E+02, 3.755730E+02, 3.759200E+02, 3.763930E+02, 3.769690E+02, 3.778120E+02, 3.787620E+02, 3.795050E+02,
 3.800830E+02, 3.805760E+02, 3.810140E+02, 3.814190E+02, 3.818260E+02, 3.822150E+02, 3.825370E+02, 3.827240E+02,
 3.828410E+02, 3.829240E+02, 3.830100E+02, 3.831090E+02, 3.831880E+02, 3.832370E+02, 3.832670E+02, 3.832960E+02,
 3.833360E+02, 3.833710E+02, 3.834580E+02, 3.835810E+02, 3.837090E+02, 3.838280E+02, 3.839130E+02, 3.839670E+02,
 3.839920E+02, 3.840120E+02, 3.840290E+02])


